from ..constants import PacketIDS
from ..utils import fmts
from . import regist_extend_function
from .basic import ExtendFunction

@regist_extend_function
class GameruleWarnings(ExtendFunction):
    def __init__(self, frame):
        super().__init__(frame)
        self.sendcommandfeedback_was_disabled = False

    def when_activate(self):
        super().when_activate()

        def on_gamerule_changed(pk: dict):
            for rule in pk["GameRules"]:
                if rule["Name"] == "sendcommandfeedback":
                    if not rule["Value"]:
                        fmts.print_war("租赁服 sendcommandfeedback 被设置为 false")
                        self.sendcommandfeedback_was_disabled = True
                    elif self.sendcommandfeedback_was_disabled:
                        fmts.print_suc("租赁服 sendcommandfeedback 已被重新设置为 true")
                        self.sendcommandfeedback_was_disabled = False
            return False

        self.frame.packet_handler.add_dict_packet_listener(
            PacketIDS.GameRulesChanged, on_gamerule_changed
        )



"插件加载器框架"

import os
import traceback
from collections.abc import Callable
from typing import TYPE_CHECKING, Any, TypeVar

from .. import utils
from ..utils import fmts
from ..mc_bytes_packet.pool import is_bytes_packet
from ..constants import (
    TOOLDELTA_CLASSIC_PLUGIN,
    TOOLDELTA_PLUGIN_DIR,
    SysStatus,
)
from .exceptions import (
    PluginAPINotFoundError,
    PluginAPIVersionError,
)
from ..constants import TextType, PacketIDS
from ..internal.packet_handler import PacketHandler
from ..internal.types import Player, Chat, InternalBroadcast, FrameExit
from ..mc_bytes_packet.base_bytes_packet import BaseBytesPacket
from ..game_utils import _set_frame
from .classic_plugin.loader import Plugin
from .classic_plugin import event_cbs as classic_plugin
from .classic_plugin import loader as classic_plugin_loader
from .basic import auto_move_plugin_dir, ON_ERROR_CB

if TYPE_CHECKING:
    from ..frame import ToolDelta

VT = TypeVar("VT")


class PluginGroup:
    "插件组类, 存放插件代码有关数据"

    def __init__(self, frame: "ToolDelta"):
        self.set_frame(frame)
        self.broadcast_listeners: dict[
            str, list[Callable[[InternalBroadcast], Any]]
        ] = {}
        self.plugin_listen_packets: set[PacketIDS] = set()
        self.plugins_api: dict[str, Plugin] = {}
        self.normal_plugin_loaded_num = 0
        self.loaded_plugin_ids = []
        self.on_err_cb = self.linked_frame.on_plugin_err

    def reload(self):
        """
        重载插件框架
        这是一个不很安全的操作, 多次 reload 后
        可能会因为一些插件线程由于底层原因无法被停止, 或者有垃圾无法被回收, 导致内存泄露等问题
        """
        self.plugins_api = {}
        self.broadcast_listeners = {}
        self.execute_frame_exit(
            FrameExit(SysStatus.NORMAL_EXIT, "normal"), self.linked_frame.on_plugin_err
        )
        classic_plugin.reload()
        fmts.print_inf("正在重新读取所有插件")
        self.load_plugins()
        self.execute_reloaded(self.linked_frame.on_plugin_err)
        fmts.print_inf("开始执行插件游戏初始化方法")
        self.execute_init(self.linked_frame.on_plugin_err)
        fmts.print_suc("重载插件已完成")

    def hook_packet_handler(self, hdl: "PacketHandler"):
        self.plugin_listen_packets = set(classic_plugin.dict_packet_funcs.keys()) | set(
            classic_plugin.bytes_packet_funcs.keys()
        )
        for pkID in self.plugin_listen_packets:

            def any_dict_pk_handler(pkt: dict, pkID=pkID):
                return self.handle_dict_packets(
                    pkID, pkt, self.linked_frame.on_plugin_err
                )

            def any_bytes_pk_handler(pkt: BaseBytesPacket, pkID=pkID):
                return self.handle_bytes_packets(
                    pkID, pkt, self.linked_frame.on_plugin_err
                )

            if is_bytes_packet(pkID):
                hdl.add_bytes_packet_listener(pkID, any_bytes_pk_handler, 0)
            else:
                hdl.add_dict_packet_listener(pkID, any_dict_pk_handler, 0)

        hdl.add_dict_packet_listener(PacketIDS.Text, self.handle_text_packet)

    def brocast_event(self, evt: InternalBroadcast) -> list[Any]:
        callback_list = []
        res = self.broadcast_listeners.get(evt.evt_name)
        if res:
            for f in res:
                res2 = f(evt)
                if res2:
                    callback_list.append(res2)
        return callback_list

    def get_plugin_api(
        self, apiName: str, min_version: tuple | None = None, force=True
    ) -> Any:
        api = self.plugins_api.get(apiName)
        if api:
            if min_version and api.version < min_version:
                raise PluginAPIVersionError(apiName, min_version, api.version)
            return api
        if force:
            raise PluginAPINotFoundError(apiName)
        return None

    def set_frame(self, frame: "ToolDelta") -> None:
        """为各个框架分发关联的系统框架"""
        self.linked_frame = frame
        _set_frame(frame)

    def load_plugins(self) -> None:
        """
        读取所有插件/重载所有插件 并对插件进行预初始化

        Raises:
            SystemExit: 读取插件出现问题
        """
        if self.linked_frame is None or self.linked_frame.on_plugin_err is None:
            raise ValueError("无法读取插件，请确保系统已初始化")
        for fdir in os.listdir(TOOLDELTA_PLUGIN_DIR):
            if fdir not in (TOOLDELTA_CLASSIC_PLUGIN,):
                auto_move_plugin_dir(fdir)
        self.loaded_plugin_ids = []
        self.normal_plugin_loaded_num = 0
        try:
            fmts.print_inf("§a正在使用 §bHiQuality §dDX§r§a 模式读取插件")
            classic_plugin_loader.read_plugins(self)
            # 主动读取类式插件监听的数据包
            for i in classic_plugin.dict_packet_funcs.keys():
                self.__add_listen_packet_id(i)
            for i in classic_plugin.bytes_packet_funcs.keys():
                self.__add_listen_packet_id(i)
            # 主动读取类式插件监听的广播事件器
            self.broadcast_listeners.update(classic_plugin.broadcast_listener)
            fmts.print_suc("所有插件读取完毕, 将进行插件初始化")
            self.execute_preload(self.linked_frame.on_plugin_err)
            fmts.print_suc(
                f"插件初始化成功, 载入 §f{self.normal_plugin_loaded_num}§a 个类式插件"
            )
        except Exception as err:
            err_str = "\n".join(traceback.format_exc().split("\n")[1:])
            fmts.print_err(f"加载插件出现问题：\n{err_str}")
            raise SystemExit from err

    def execute_preload(self, onerr: ON_ERROR_CB) -> None:
        """执行插件的二次初始化方法

        Args:
            onerr (Callable[[str, Exception, str], None], optional): 插件出错时的处理方法。Defaults to non_func.

        Raises:
            SystemExit: 缺少前置
            SystemExit: 前置版本过低
        """
        classic_plugin.execute_preload(onerr)

    def execute_init(self, onerr: ON_ERROR_CB) -> None:
        """执行插件的连接游戏后初始化方法

        Args:
            onerr (Callable[[str, Exception, str], None], optional): 插件出错时的处理方法
        """
        classic_plugin.execute_active(onerr)

    def execute_player_join(self, player: Player, onerr: ON_ERROR_CB) -> None:
        """执行玩家加入的方法

        Args:
            player (str): 玩家
            onerr (Callable[[str, Exception, str], None], optional): q 插件出错时的处理方法
        """
        classic_plugin.execute_player_join(player, onerr)

    def execute_chat(
        self,
        chat: Chat,
        onerr: ON_ERROR_CB,
    ) -> None:
        """执行玩家消息的方法

        Args:
            player (str): 玩家
            msg (str): 消息
            onerr (Callable[[str, Exception, str], None], optional): 插件出错时的处理方法
        """
        classic_plugin.execute_chat(chat, onerr)

    def execute_player_leave(self, player: Player, onerr: ON_ERROR_CB) -> None:
        """执行玩家离开的方法

        Args:
            player (str): 玩家
            onerr (Callable[[str, Exception, str], None], optional): 插件出错时的处理方法
        """
        classic_plugin.execute_player_leave(player, onerr)

    def execute_frame_exit(self, evt: FrameExit, onerr: ON_ERROR_CB):
        """执行框架退出的方法

        Args:
            onerr (Callable[[str, Exception, str], None], optional): 插件出错时的处理方法
        """
        classic_plugin.execute_frame_exit(evt, onerr)

    def execute_reloaded(self, onerr: ON_ERROR_CB):
        """执行插件重载的方法

        Args:
            onerr (Callable[[str, Exception, str], None], optional): 插件出错时的处理方法
        """
        classic_plugin.execute_reloaded(onerr)

    def handle_dict_packets(
        self, pktID: PacketIDS, pkt: dict, onerr: ON_ERROR_CB
    ) -> bool:
        """处理字典型数据包的监听器

        Args:
            pktID (int): 数据包 ID
            pkt (dict): 数据包

        Returns:
            bool: 是否处理成功
        """
        blocking = classic_plugin.execute_dict_packet_funcs(pktID, pkt, onerr)
        return blocking

    def handle_bytes_packets(
        self, pktID: PacketIDS, pkt: BaseBytesPacket, onerr: ON_ERROR_CB
    ) -> bool:
        """处理二进制数据包的监听器

        Args:
            pktID (int): 数据包 ID
            pkt (BaseBytesPacket): 数据包

        Returns:
            bool: 是否处理成功
        """
        blocking = classic_plugin.execute_bytes_packet_funcs(pktID, pkt, onerr)
        return blocking

    def handle_text_packet(self, pkt: dict):
        match pkt["TextType"]:
            case TextType.TextTypeTranslation:
                if pkt["Message"] == "§e%multiplayer.player.joined":
                    playername = pkt["Parameters"][0]
                    if player := self.linked_frame.players_maintainer.getPlayerByName(
                        playername
                    ):
                        self.execute_player_join(
                            player, self.linked_frame.on_plugin_err
                        )
                    else:
                        fmts.print_war(f"玩家 {playername} 未找到")
                elif pkt["Message"] == "§e%multiplayer.player.left":
                    playername = pkt["Parameters"][0]
                    if player := self.linked_frame.players_maintainer.getPlayerByName(
                        playername
                    ):
                        self.execute_player_leave(
                            player, self.linked_frame.on_plugin_err
                        )
                    else:
                        fmts.print_war(f"玩家 {playername} 未找到")
            case _:
                playername, message, ensurePlayer = (
                    utils.get_playername_and_msg_from_text_packet(
                        self.linked_frame, pkt
                    )
                )
                if playername is not None and message is not None and ensurePlayer:
                    if player := self.linked_frame.players_maintainer.getPlayerByName(
                        playername
                    ):
                        chat = Chat(player, message)
                        self.execute_chat(chat, self.linked_frame.on_plugin_err)
        return False

    help = staticmethod(classic_plugin_loader.help)

    def __add_listen_packet_id(self, packetType: PacketIDS) -> None:
        """添加数据包监听，仅在系统内部使用

        Args:
            packetType (int): 数据包 ID

        Raises:
            ValueError: 无法添加数据包监听，请确保已经加载了系统组件
        """
        self.plugin_listen_packets.add(packetType)



import struct
import numpy

from dataclasses import dataclass, field
from io import BytesIO
from tooldelta.internal.launch_cli.neo_libs.blob_hash.packet.define import (
    HashWithPosition,
    PayloadByHash,
)


@dataclass
class SetHolderRequest:
    packet_name: str = "blob-hash-set-holder-request"

    def encode(self) -> bytes:
        return b""


@dataclass
class SetHolderResponse:
    success_states: bool = False
    holder_name: str = ""
    packet_name: str = "blob-hash-set-holder-response"

    def decode(self, bs: bytes | None):
        if bs is None:
            return
        reader = BytesIO(bs)
        self.success_states = bool(reader.read(1)[0])
        self.holder_name = reader.read(struct.unpack("<h", reader.read(2))[0]).decode(
            encoding="utf-8"
        )


@dataclass
class GetHashPayload:
    hashes: list[HashWithPosition] = field(default_factory=lambda: [])
    packet_name: str = "blob-hash-get-hash-payload"

    def encode(self) -> bytes:
        writer = BytesIO()
        writer.write(struct.pack("<H", len(self.hashes)))
        for i in self.hashes:
            i.encode(writer)
        return writer.getvalue()


@dataclass
class GetHashPayloadResponse:
    payload: list[PayloadByHash] = field(default_factory=lambda: [])
    packet_name: str = "blob-hash-get-hash-payload-response"

    def decode(self, bs: bytes | None):
        if bs is None:
            return
        reader = BytesIO(bs)
        for _ in range(struct.unpack("<H", reader.read(2))[0]):
            p = PayloadByHash()
            p.decode(reader)
            self.payload.append(p)


@dataclass
class ClientQueryDiskHashExist:
    hashes: list[HashWithPosition] = field(default_factory=lambda: [])
    packet_name: str = "blob-hash-client-query-disk-hash-exist"

    def encode(self) -> bytes:
        writer = BytesIO()
        writer.write(struct.pack("<H", len(self.hashes)))
        for i in self.hashes:
            i.encode(writer)
        return writer.getvalue()


@dataclass
class ClientQueryDiskHashExistResponse:
    states: numpy.ndarray = field(
        default_factory=lambda: numpy.array([], dtype=numpy.bool)
    )
    packet_name: str = "blob-hash-client-query-disk-hash-exist-response"

    def decode(self, bs: bytes | None):
        if bs is None:
            return
        reader = BytesIO(bs)
        self.states = numpy.frombuffer(
            reader.read(struct.unpack("<H", reader.read(2))[0]), dtype=numpy.bool
        )


@dataclass
class ClientGetDiskHashPayload:
    hashes: list[HashWithPosition] = field(default_factory=lambda: [])
    packet_name: str = "blob-hash-client-get-disk-hash-payload"

    def encode(self) -> bytes:
        writer = BytesIO()
        writer.write(struct.pack("<H", len(self.hashes)))
        for i in self.hashes:
            i.encode(writer)
        return writer.getvalue()


@dataclass
class ClientGetDiskHashPayloadResponse:
    payload: list[PayloadByHash] = field(default_factory=lambda: [])
    packet_name: str = "blob-hash-client-get-disk-hash-payload-response"

    def decode(self, bs: bytes | None):
        if bs is None:
            return
        reader = BytesIO(bs)
        for _ in range(struct.unpack("<H", reader.read(2))[0]):
            p = PayloadByHash()
            p.decode(reader)
            self.payload.append(p)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: proto/playerkit.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'proto/playerkit.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import response_pb2 as proto_dot_response__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15proto/playerkit.proto\x12\x17\x66\x61teark.proto.playerkit\x1a\x14proto/response.proto\"\x1c\n\x1aGetAllOnlinePlayersRequest\"&\n\x16GetPlayerByNameRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"&\n\x16GetPlayerByUUIDRequest\x12\x0c\n\x04uuid\x18\x01 \x01(\t\",\n\x18ReleaseBindPlayerRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"(\n\x14GetPlayerNameRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"2\n\x1eGetPlayerEntityUniqueIDRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"-\n\x19GetPlayerLoginTimeRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"2\n\x1eGetPlayerPlatformChatIDRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"1\n\x1dGetPlayerBuildPlatformRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"*\n\x16GetPlayerSkinIDRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\",\n\x18GetPlayerCanBuildRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\";\n\x18SetPlayerCanBuildRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\r\n\x05\x61llow\x18\x02 \x01(\x08\"*\n\x16GetPlayerCanDigRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"9\n\x16SetPlayerCanDigRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\r\n\x05\x61llow\x18\x02 \x01(\x08\"7\n#GetPlayerCanDoorsAndSwitchesRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"F\n#SetPlayerCanDoorsAndSwitchesRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\r\n\x05\x61llow\x18\x02 \x01(\x08\"5\n!GetPlayerCanOpenContainersRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"D\n!SetPlayerCanOpenContainersRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\r\n\x05\x61llow\x18\x02 \x01(\x08\"4\n GetPlayerCanAttackPlayersRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"C\n SetPlayerCanAttackPlayersRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\r\n\x05\x61llow\x18\x02 \x01(\x08\"1\n\x1dGetPlayerCanAttackMobsRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"@\n\x1dSetPlayerCanAttackMobsRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\r\n\x05\x61llow\x18\x02 \x01(\x08\"7\n#GetPlayerCanOperatorCommandsRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"F\n#SetPlayerCanOperatorCommandsRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\r\n\x05\x61llow\x18\x02 \x01(\x08\"/\n\x1bGetPlayerCanTeleportRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\">\n\x1bSetPlayerCanTeleportRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\r\n\x05\x61llow\x18\x02 \x01(\x08\"6\n\"GetPlayerStatusInvulnerableRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"0\n\x1cGetPlayerStatusFlyingRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"0\n\x1cGetPlayerStatusMayFlyRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\",\n\x18GetPlayerDeviceIDRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"3\n\x1fGetPlayerEntityRuntimeIDRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"2\n\x1eGetPlayerEntityMetadataRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"(\n\x14GetPlayerIsOPRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"*\n\x16GetPlayerOnlineRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\"6\n\x15SendPlayerChatRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\x0b\n\x03msg\x18\x02 \x01(\t\"9\n\x18SendPlayerRawChatRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\x0b\n\x03msg\x18\x02 \x01(\t\"L\n\x16SendPlayerTitleRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\r\n\x05title\x18\x02 \x01(\t\x12\x11\n\tsub_title\x18\x03 \x01(\t\"B\n\x1aSendPlayerActionBarRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\x12\n\naction_bar\x18\x02 \x01(\t\"M\n#InterceptPlayerJustNextInputRequest\x12\x10\n\x08uuid_str\x18\x01 \x01(\t\x12\x14\n\x0cretriever_id\x18\x02 \x01(\t2\xab%\n\x10PlayerKitService\x12s\n\x13GetAllOnlinePlayers\x12\x33.fateark.proto.playerkit.GetAllOnlinePlayersRequest\x1a\'.fateark.proto.response.GeneralResponse\x12k\n\x0fGetPlayerByName\x12/.fateark.proto.playerkit.GetPlayerByNameRequest\x1a\'.fateark.proto.response.GeneralResponse\x12k\n\x0fGetPlayerByUUID\x12/.fateark.proto.playerkit.GetPlayerByUUIDRequest\x1a\'.fateark.proto.response.GeneralResponse\x12o\n\x11ReleaseBindPlayer\x12\x31.fateark.proto.playerkit.ReleaseBindPlayerRequest\x1a\'.fateark.proto.response.GeneralResponse\x12g\n\rGetPlayerName\x12-.fateark.proto.playerkit.GetPlayerNameRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x80\x01\n\x17GetPlayerEntityUniqueID\x12\x37.fateark.proto.playerkit.GetPlayerEntityUniqueIDRequest\x1a,.fateark.proto.response.GeneralInt64Response\x12v\n\x12GetPlayerLoginTime\x12\x32.fateark.proto.playerkit.GetPlayerLoginTimeRequest\x1a,.fateark.proto.response.GeneralInt64Response\x12{\n\x17GetPlayerPlatformChatID\x12\x37.fateark.proto.playerkit.GetPlayerPlatformChatIDRequest\x1a\'.fateark.proto.response.GeneralResponse\x12~\n\x16GetPlayerBuildPlatform\x12\x36.fateark.proto.playerkit.GetPlayerBuildPlatformRequest\x1a,.fateark.proto.response.GeneralInt32Response\x12k\n\x0fGetPlayerSkinID\x12/.fateark.proto.playerkit.GetPlayerSkinIDRequest\x1a\'.fateark.proto.response.GeneralResponse\x12s\n\x11GetPlayerCanBuild\x12\x31.fateark.proto.playerkit.GetPlayerCanBuildRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12o\n\x11SetPlayerCanBuild\x12\x31.fateark.proto.playerkit.SetPlayerCanBuildRequest\x1a\'.fateark.proto.response.GeneralResponse\x12o\n\x0fGetPlayerCanDig\x12/.fateark.proto.playerkit.GetPlayerCanDigRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12k\n\x0fSetPlayerCanDig\x12/.fateark.proto.playerkit.SetPlayerCanDigRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x89\x01\n\x1cGetPlayerCanDoorsAndSwitches\x12<.fateark.proto.playerkit.GetPlayerCanDoorsAndSwitchesRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12\x85\x01\n\x1cSetPlayerCanDoorsAndSwitches\x12<.fateark.proto.playerkit.SetPlayerCanDoorsAndSwitchesRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x85\x01\n\x1aGetPlayerCanOpenContainers\x12:.fateark.proto.playerkit.GetPlayerCanOpenContainersRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12\x81\x01\n\x1aSetPlayerCanOpenContainers\x12:.fateark.proto.playerkit.SetPlayerCanOpenContainersRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x83\x01\n\x19GetPlayerCanAttackPlayers\x12\x39.fateark.proto.playerkit.GetPlayerCanAttackPlayersRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12\x7f\n\x19SetPlayerCanAttackPlayers\x12\x39.fateark.proto.playerkit.SetPlayerCanAttackPlayersRequest\x1a\'.fateark.proto.response.GeneralResponse\x12}\n\x16GetPlayerCanAttackMobs\x12\x36.fateark.proto.playerkit.GetPlayerCanAttackMobsRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12y\n\x16SetPlayerCanAttackMobs\x12\x36.fateark.proto.playerkit.SetPlayerCanAttackMobsRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x89\x01\n\x1cGetPlayerCanOperatorCommands\x12<.fateark.proto.playerkit.GetPlayerCanOperatorCommandsRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12\x85\x01\n\x1cSetPlayerCanOperatorCommands\x12<.fateark.proto.playerkit.SetPlayerCanOperatorCommandsRequest\x1a\'.fateark.proto.response.GeneralResponse\x12y\n\x14GetPlayerCanTeleport\x12\x34.fateark.proto.playerkit.GetPlayerCanTeleportRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12y\n\x14SetPlayerCanTeleport\x12\x34.fateark.proto.playerkit.SetPlayerCanTeleportRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12\x87\x01\n\x1bGetPlayerStatusInvulnerable\x12;.fateark.proto.playerkit.GetPlayerStatusInvulnerableRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12{\n\x15GetPlayerStatusFlying\x12\x35.fateark.proto.playerkit.GetPlayerStatusFlyingRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12{\n\x15GetPlayerStatusMayFly\x12\x35.fateark.proto.playerkit.GetPlayerStatusMayFlyRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12o\n\x11GetPlayerDeviceID\x12\x31.fateark.proto.playerkit.GetPlayerDeviceIDRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x83\x01\n\x18GetPlayerEntityRuntimeID\x12\x38.fateark.proto.playerkit.GetPlayerEntityRuntimeIDRequest\x1a-.fateark.proto.response.GeneralUint64Response\x12{\n\x17GetPlayerEntityMetadata\x12\x37.fateark.proto.playerkit.GetPlayerEntityMetadataRequest\x1a\'.fateark.proto.response.GeneralResponse\x12k\n\rGetPlayerIsOP\x12-.fateark.proto.playerkit.GetPlayerIsOPRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12o\n\x0fGetPlayerOnline\x12/.fateark.proto.playerkit.GetPlayerOnlineRequest\x1a+.fateark.proto.response.GeneralBoolResponse\x12i\n\x0eSendPlayerChat\x12..fateark.proto.playerkit.SendPlayerChatRequest\x1a\'.fateark.proto.response.GeneralResponse\x12o\n\x11SendPlayerRawChat\x12\x31.fateark.proto.playerkit.SendPlayerRawChatRequest\x1a\'.fateark.proto.response.GeneralResponse\x12k\n\x0fSendPlayerTitle\x12/.fateark.proto.playerkit.SendPlayerTitleRequest\x1a\'.fateark.proto.response.GeneralResponse\x12s\n\x13SendPlayerActionBar\x12\x33.fateark.proto.playerkit.SendPlayerActionBarRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x85\x01\n\x1cInterceptPlayerJustNextInput\x12<.fateark.proto.playerkit.InterceptPlayerJustNextInputRequest\x1a\'.fateark.proto.response.GeneralResponseB#Z!./proto/gen/playerkit;playerkitpbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.playerkit_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z!./proto/gen/playerkit;playerkitpb'
  _globals['_GETALLONLINEPLAYERSREQUEST']._serialized_start=72
  _globals['_GETALLONLINEPLAYERSREQUEST']._serialized_end=100
  _globals['_GETPLAYERBYNAMEREQUEST']._serialized_start=102
  _globals['_GETPLAYERBYNAMEREQUEST']._serialized_end=140
  _globals['_GETPLAYERBYUUIDREQUEST']._serialized_start=142
  _globals['_GETPLAYERBYUUIDREQUEST']._serialized_end=180
  _globals['_RELEASEBINDPLAYERREQUEST']._serialized_start=182
  _globals['_RELEASEBINDPLAYERREQUEST']._serialized_end=226
  _globals['_GETPLAYERNAMEREQUEST']._serialized_start=228
  _globals['_GETPLAYERNAMEREQUEST']._serialized_end=268
  _globals['_GETPLAYERENTITYUNIQUEIDREQUEST']._serialized_start=270
  _globals['_GETPLAYERENTITYUNIQUEIDREQUEST']._serialized_end=320
  _globals['_GETPLAYERLOGINTIMEREQUEST']._serialized_start=322
  _globals['_GETPLAYERLOGINTIMEREQUEST']._serialized_end=367
  _globals['_GETPLAYERPLATFORMCHATIDREQUEST']._serialized_start=369
  _globals['_GETPLAYERPLATFORMCHATIDREQUEST']._serialized_end=419
  _globals['_GETPLAYERBUILDPLATFORMREQUEST']._serialized_start=421
  _globals['_GETPLAYERBUILDPLATFORMREQUEST']._serialized_end=470
  _globals['_GETPLAYERSKINIDREQUEST']._serialized_start=472
  _globals['_GETPLAYERSKINIDREQUEST']._serialized_end=514
  _globals['_GETPLAYERCANBUILDREQUEST']._serialized_start=516
  _globals['_GETPLAYERCANBUILDREQUEST']._serialized_end=560
  _globals['_SETPLAYERCANBUILDREQUEST']._serialized_start=562
  _globals['_SETPLAYERCANBUILDREQUEST']._serialized_end=621
  _globals['_GETPLAYERCANDIGREQUEST']._serialized_start=623
  _globals['_GETPLAYERCANDIGREQUEST']._serialized_end=665
  _globals['_SETPLAYERCANDIGREQUEST']._serialized_start=667
  _globals['_SETPLAYERCANDIGREQUEST']._serialized_end=724
  _globals['_GETPLAYERCANDOORSANDSWITCHESREQUEST']._serialized_start=726
  _globals['_GETPLAYERCANDOORSANDSWITCHESREQUEST']._serialized_end=781
  _globals['_SETPLAYERCANDOORSANDSWITCHESREQUEST']._serialized_start=783
  _globals['_SETPLAYERCANDOORSANDSWITCHESREQUEST']._serialized_end=853
  _globals['_GETPLAYERCANOPENCONTAINERSREQUEST']._serialized_start=855
  _globals['_GETPLAYERCANOPENCONTAINERSREQUEST']._serialized_end=908
  _globals['_SETPLAYERCANOPENCONTAINERSREQUEST']._serialized_start=910
  _globals['_SETPLAYERCANOPENCONTAINERSREQUEST']._serialized_end=978
  _globals['_GETPLAYERCANATTACKPLAYERSREQUEST']._serialized_start=980
  _globals['_GETPLAYERCANATTACKPLAYERSREQUEST']._serialized_end=1032
  _globals['_SETPLAYERCANATTACKPLAYERSREQUEST']._serialized_start=1034
  _globals['_SETPLAYERCANATTACKPLAYERSREQUEST']._serialized_end=1101
  _globals['_GETPLAYERCANATTACKMOBSREQUEST']._serialized_start=1103
  _globals['_GETPLAYERCANATTACKMOBSREQUEST']._serialized_end=1152
  _globals['_SETPLAYERCANATTACKMOBSREQUEST']._serialized_start=1154
  _globals['_SETPLAYERCANATTACKMOBSREQUEST']._serialized_end=1218
  _globals['_GETPLAYERCANOPERATORCOMMANDSREQUEST']._serialized_start=1220
  _globals['_GETPLAYERCANOPERATORCOMMANDSREQUEST']._serialized_end=1275
  _globals['_SETPLAYERCANOPERATORCOMMANDSREQUEST']._serialized_start=1277
  _globals['_SETPLAYERCANOPERATORCOMMANDSREQUEST']._serialized_end=1347
  _globals['_GETPLAYERCANTELEPORTREQUEST']._serialized_start=1349
  _globals['_GETPLAYERCANTELEPORTREQUEST']._serialized_end=1396
  _globals['_SETPLAYERCANTELEPORTREQUEST']._serialized_start=1398
  _globals['_SETPLAYERCANTELEPORTREQUEST']._serialized_end=1460
  _globals['_GETPLAYERSTATUSINVULNERABLEREQUEST']._serialized_start=1462
  _globals['_GETPLAYERSTATUSINVULNERABLEREQUEST']._serialized_end=1516
  _globals['_GETPLAYERSTATUSFLYINGREQUEST']._serialized_start=1518
  _globals['_GETPLAYERSTATUSFLYINGREQUEST']._serialized_end=1566
  _globals['_GETPLAYERSTATUSMAYFLYREQUEST']._serialized_start=1568
  _globals['_GETPLAYERSTATUSMAYFLYREQUEST']._serialized_end=1616
  _globals['_GETPLAYERDEVICEIDREQUEST']._serialized_start=1618
  _globals['_GETPLAYERDEVICEIDREQUEST']._serialized_end=1662
  _globals['_GETPLAYERENTITYRUNTIMEIDREQUEST']._serialized_start=1664
  _globals['_GETPLAYERENTITYRUNTIMEIDREQUEST']._serialized_end=1715
  _globals['_GETPLAYERENTITYMETADATAREQUEST']._serialized_start=1717
  _globals['_GETPLAYERENTITYMETADATAREQUEST']._serialized_end=1767
  _globals['_GETPLAYERISOPREQUEST']._serialized_start=1769
  _globals['_GETPLAYERISOPREQUEST']._serialized_end=1809
  _globals['_GETPLAYERONLINEREQUEST']._serialized_start=1811
  _globals['_GETPLAYERONLINEREQUEST']._serialized_end=1853
  _globals['_SENDPLAYERCHATREQUEST']._serialized_start=1855
  _globals['_SENDPLAYERCHATREQUEST']._serialized_end=1909
  _globals['_SENDPLAYERRAWCHATREQUEST']._serialized_start=1911
  _globals['_SENDPLAYERRAWCHATREQUEST']._serialized_end=1968
  _globals['_SENDPLAYERTITLEREQUEST']._serialized_start=1970
  _globals['_SENDPLAYERTITLEREQUEST']._serialized_end=2046
  _globals['_SENDPLAYERACTIONBARREQUEST']._serialized_start=2048
  _globals['_SENDPLAYERACTIONBARREQUEST']._serialized_end=2114
  _globals['_INTERCEPTPLAYERJUSTNEXTINPUTREQUEST']._serialized_start=2116
  _globals['_INTERCEPTPLAYERJUSTNEXTINPUTREQUEST']._serialized_end=2193
  _globals['_PLAYERKITSERVICE']._serialized_start=2196
  _globals['_PLAYERKITSERVICE']._serialized_end=6975
# @@protoc_insertion_point(module_scope)



"启动页面"

import signal
import os
import time
import traceback

from random import choice
from .utils import fmts, sys_args
from .constants import TOOLDELTA_LOGO_mode
from .internal.config_loader import ConfigLoader
from .plugin_manager import plugin_manager
from .plugin_market import market
from .starter import start_tool_delta, init_cfg_only
from .version import get_tool_delta_version


def signal_handler(*_) -> None:
    """排除信号中断"""
    ...


signal.signal(signal.SIGINT, signal_handler)


def client_title() -> None:
    """选择启动模式"""
    try:
        launch_args = sys_args.sys_args_to_dict()
        if opt_str := launch_args.get("optadd"):
            more_opts = {
                str(i + 7): (k, v)
                for (i, (k, v)) in enumerate(sys_args.parse_addopt(opt_str).items())
            }
        else:
            more_opts = {}
        if "h" in launch_args or "help" in launch_args:
            sys_args.print_help()
            return
        elif "v" in launch_args or "version" in launch_args:
            print(".".join(str(i) for i in get_tool_delta_version()))
            return
        is_faststart = os.path.isfile("快速启动.sig")
        if launch_args.get("l"):
            if not isinstance(launch_args["l"], str):
                raise ValueError("启动模式参数不合法")
            r = launch_args["l"]
        elif is_faststart:
            fmts.clean_print("§a快速启动功能已打开, 将跳过选择界面")
            fmts.clean_print("删除本地的 §f快速启动.sig§r 文件即可取消快速启动功能")
            time.sleep(0.5)
            start_tool_delta()
            return
        else:
            text=choice(TOOLDELTA_LOGO_mode)
            if text[0]==0:
                print(fmts.clean_fmt(text[1]))
            elif text[0]==1:
                print(fmts.print_gradient(text[1],text[2],text[3]))
            if "title" in sys_args.sys_args_to_dict():
                fmts.clean_print(launch_args["title"] or "")
            fmts.clean_print(
                "§a请选择启动模式§6(使用启动参数 -l <启动模式> 可以跳过该页面):"
            )
            fmts.clean_print("1 - §b启动 ToolDelta")
            fmts.clean_print("2 - §d打开插件管理器")
            fmts.clean_print("3 - §d打开插件市场")
            fmts.clean_print("4 - §6初始化所有插件配置")
            fmts.clean_print("5 - §a修改启动配置")
            fmts.clean_print("6 - §c开启直接启动模式")
            for i, (opt_name, _) in enumerate(more_opts.values()):
                fmts.clean_print(f"{i + 7} - {opt_name}")
            fmts.clean_print("q - §7退出")
            r = input("请选择：").strip()
        match r:
            case "1":
                start_tool_delta()
            case "2":
                plugin_manager.manage_plugins()
            case "3":
                market.enter_plugin_market()
            case "4":
                init_cfg_only()
            case "5":
                ConfigLoader.change_config()
            case "6":
                open("快速启动.sig", "wb").close()
                fmts.clean_print("§a快速启动模式已开启")
                fmts.clean_print(
                    "§6删除本地的 §f快速启动.sig§6 文件即可取消快速启动功能"
                )
            case "q":
                fmts.clean_print("§aToolDelta 已退出.")
            case _:
                if r in more_opts.keys():
                    os.system(more_opts[r][1])
                else:
                    fmts.clean_print("§c不合法的启动模式: " + r)
    except (EOFError, SystemExit):
        pass
    except Exception:
        fmts.print_err("ToolDelta 运行过程中出现问题：" + traceback.format_exc())
        input(fmts.fmt_info("按回车键退出..", "§c 报错 §r"))



from collections.abc import Callable
from threading import Lock

from . import fmts
from .tooldelta_thread import ThreadExit


class ChatbarLock:
    """
    聊天栏锁, 用于防止玩家同时开启多个聊天栏对话

    调用了该锁的所有代码, 在另一个进程使用该锁的时候, 尝试调用其他锁会导致进程直接退出, 直到此锁退出为止

    示例(以类式插件为例):
    ```python
    class MyPlugin(Plugin):
        ...
        def on_player_message(self, player: str, msg: str):
            with ChatbarLock(player):
                # 如果玩家处在另一个on_player_message进程 (锁环境) 中
                # 则在上面就会直接引发 SystemExit
                ...
    ```
    """

    __slots__ = ("oth_cb", "player")

    def __init__(self, player: str, oth_cb: Callable[[str], None] = lambda _: None):
        self.player = player
        self.oth_cb = oth_cb

    def __enter__(self):
        if self.player in players_in_chatbar_lock:
            self.oth_cb(self.player)
            fmts.print_war(f"玩家 {self.player} 的线程锁正在锁定状态")
            raise ThreadExit
        if self.player not in players_in_chatbar_lock:
            players_in_chatbar_lock.append(self.player)

    def __exit__(self, *_):
        players_in_chatbar_lock.remove(self.player)


class BotActionLock:
    """
    机器人行动锁, 用于防止多个线程同时使用机器人与游戏交互, 造成错误
    """

    __slots__ = ("name", "when_locked")

    def __init__(self, lock_name: str, when_locked: Callable[[], None] | None = None):
        """
        Args:
            lock_name (str): 行动锁名

        Args:
            lock_name (str): _description_
            when_locked (Callable[[], None] | None, optional): _description_. Defaults to None.
        """
        global bot_action_lock_current_name
        bot_action_lock_current_name = lock_name
        self.name = lock_name
        self.when_locked = when_locked

    def __enter__(self):
        locked = bot_action_lock.locked()
        if locked:
            fmts.print_war(f"[BotAction] {self.name} 与 {bot_action_lock_current_name} 冲突")
        if locked and self.when_locked is not None:
            self.when_locked()
        else:
            bot_action_lock.acquire()

    def __exit__(self, *_):
        if bot_action_lock.locked():
            bot_action_lock.release()


players_in_chatbar_lock: list[str] = []
bot_action_lock = Lock()
bot_action_lock_current_name = ""



from . import (
    tooldelta_thread,
    tempjson,
    timer_events,
    fmts,
    logger
)

def safe_close():
    """安全关闭: 保存JSON配置文件和关闭所有定时任务"""
    timer_events.timer_event_stop.set()
    timer_events.timer_events_clear()
    fmts.print_inf("正在强制中断 ToolDeltThread..")
    tooldelta_thread.force_stop_normal_threads()
    fmts.print_inf("正在保存数据文件..")
    tempjson.save_all()
    logger.publicLogger.reset()



import os

from . import fmts

def if_token() -> None:
    """检查路径下是否有 fbtoken，没有就提示输入

    Raises:
        SystemExit: 未输入 fbtoken
    """
    if not os.path.isfile("fbtoken"):
        fmts.print_inf(
            "请到对应的验证服务器官网下载 FBToken，并放在本目录中，或者在下面输入 fbtoken"
        )
        fbtoken = input(fmts.fmt_info("请输入 fbtoken: ", "§b 输入 "))
        if fbtoken:
            with open("fbtoken", "w", encoding="utf-8") as f:
                f.write(fbtoken)
        else:
            fmts.print_err("未输入 fbtoken, 无法继续")
            raise SystemExit


def fbtokenFix():
    """修复 fbtoken 里的换行符"""
    with open("fbtoken", encoding="utf-8") as file:
        token = file.read()
        if "\n" in token:
            fmts.print_war("fbtoken 里有换行符，会造成 fb 登录失败，已自动修复")
            with open("fbtoken", "w", encoding="utf-8") as file2:
                file2.write(token.replace("\n", ""))



"获取 ToolDelta 的版本号"

VERSION = tuple[int, int, int]


class SystemVersionException(SystemError):
    """系统版本异常"""


def get_tool_delta_version() -> VERSION:
    """获取 ToolDelta 的版本号

    Returns:
        VERSION: 版本号
    """
    # 由 Github 自动同步
    return (1, 0, 8)


def check_tooldelta_version(need_vers: VERSION):
    """检查 ToolDelta 系统的版本

    Args:
        need_vers (VERSION): 需要的版本

    Raises:
        self.linked_frame.SystemVersionException: 该组件需要的 ToolDelta 系统版本
    """
    if need_vers >= get_tool_delta_version():
        raise SystemVersionException(
            f"ToolDelta 版本不匹配, 需要最低 {version_str(need_vers)} 版本, 当前 {version_str(get_tool_delta_version())}"
        )


def version_str(ver: VERSION):
    return ".".join(map(str, ver))



"插件加载主模块"

import json
import os
import shutil
from typing import Any
from collections.abc import Callable

from tooldelta.utils import fmts
from tooldelta.constants import (
    PLUGIN_TYPE_MAPPING,
    TOOLDELTA_CLASSIC_PLUGIN,
    TOOLDELTA_PLUGIN_DIR,
)


def non_func(*_) -> None:
    """空函数"""
    return None


ON_ERROR_CB = Callable[[str, Exception], None]


class PluginRegData:
    """插件注册数据"""

    def __init__(
        self,
        name: str,
        plugin_data: dict | None = None,
        is_registered=True,
        is_enabled=True,
    ):
        """插件注册数据

        Args:
            name (str): 插件名
            plugin_data (dict | None, optional): 插件数据
            is_registered (bool, optional): 是否已注册
            is_enabled (bool, optional): 是否启用
        """
        if plugin_data is None:
            plugin_data = {}
        self.name: str = name
        if isinstance(plugin_data.get("version"), str):
            self.version: tuple = tuple(
                int(i) for i in plugin_data.get("version", "0.0.0").split(".")
            )
        else:
            self.version = plugin_data.get("version", (0, 0, 0))
        #  以下的方法或许存在危险
        self.author: str = plugin_data.get("author", "unknown")
        self.plugin_type: str = plugin_data.get("plugin-type", "unknown")
        self.description: str = plugin_data.get("description", "")
        self.pre_plugins: dict[str, str] = plugin_data.get("pre-plugins", {})
        self.plugin_id = plugin_data.get("plugin-id", "???")
        self.is_registered = is_registered
        self.is_deleted = False
        if plugin_data.get("enabled") is not None:
            self.is_enabled = plugin_data["enabled"]
        else:
            self.is_enabled = is_enabled

    @property
    def dir(self):
        name = os.path.join(
            str(TOOLDELTA_PLUGIN_DIR),
            {
                "classic": TOOLDELTA_CLASSIC_PLUGIN,
            }[self.plugin_type],
            self.name,
        )
        return name + ("" if self.is_enabled else "+disabled")

    def dump(self) -> dict[str, Any]:
        """转储数据"""
        return {
            "author": self.author,
            "version": ".".join([str(i) for i in self.version]),
            "plugin-type": self.plugin_type,
            "description": self.description,
            "pre-plugins": self.pre_plugins,
            "plugin-id": self.plugin_id,
            "enabled": self.is_enabled,
        }

    @property
    def version_str(self) -> str:
        """版本字符串

        Returns:
            str: 版本字符串
        """
        return ".".join(str(i) for i in self.version)

    @property
    def plugin_type_str(self) -> str:
        """插件类型字符串

        Returns:
            str: 插件类型字符串
        """
        return {
            "classic": "主类式",
            "unknown": "未知类型",
        }.get(self.plugin_type, "未知类型")


class PluginsPackage:
    """插件整合包"""

    def __init__(self, name: str, pack_datas: dict):
        self.name = name
        self.author: str = pack_datas["author"]
        self.version: str = pack_datas["version"]
        self.description: str = pack_datas["description"]
        self.plugin_ids: list[str] = pack_datas["plugin-ids"]


def plugin_is_enabled(pname: str) -> bool:
    """
    插件是否被启用

    Args:
        pname (str): 插件文件夹名

    Returns:
        bool: 是否启用
    """
    return not pname.endswith("+disabled")


def auto_move_plugin_dir(fdname: str):
    """
    自动尝试移动一个总插件文件夹下可能为插件文件夹的文件夹至正确的插件路径
    move "插件文件/{plugin_dir}" -> "插件文件/{plugin_type}/{plugin_dir}

    Args:
        fdname (str): 插件文件夹名
    """
    data_path = os.path.join(TOOLDELTA_PLUGIN_DIR, fdname, "datas.json")
    if os.path.isfile(data_path):
        try:
            with open(data_path, encoding="utf-8") as f:
                plugin_data_json = json.load(f)
                p_type = plugin_data_json["plugin-type"]
                if p_type not in PLUGIN_TYPE_MAPPING:
                    fmts.print_war(f"无法识别插件 {fdname} 的类型，跳过")
                    return
            shutil.move(
                os.path.join(TOOLDELTA_PLUGIN_DIR, fdname),
                os.path.join(TOOLDELTA_PLUGIN_DIR, PLUGIN_TYPE_MAPPING[p_type]),
            )
            fmts.print_suc(
                f"已将插件 {fdname} 智能移动至 {PLUGIN_TYPE_MAPPING[p_type]} 插件文件夹内"
            )
        except Exception as err:
            fmts.print_err(f"智能移动插件文件夹 {fdname} 出错：{err}")



import importlib
import os
import sys
import traceback
from typing import TYPE_CHECKING, TypeVar

from ... import utils
from ...utils import cfg, fmts
from ...constants import (
    TOOLDELTA_PLUGIN_DIR,
    TOOLDELTA_CLASSIC_PLUGIN,
)
from ...version import SystemVersionException
from ..basic import plugin_is_enabled
from ..exceptions import NotValidPluginError
from . import event_cbs
from .plugin_cls import Plugin

if TYPE_CHECKING:
    from ... import ToolDelta
    from ...plugin_load.plugins import PluginGroup

loaded_plugin_modules = []
__cached_frame: "ToolDelta | None" = None

PLUGIN_CLS = TypeVar("PLUGIN_CLS", bound=Plugin)


# TODO: 会存储已删除的插件模块, 可能导致内存泄漏
def plugin_entry(
    plugin_cls: type[PLUGIN_CLS],
    api_name: str | list[str] = [],
    api_version: cfg.VERSION | None = None,
) -> PLUGIN_CLS:
    """
    实例化 ToolDelta 类式插件的主类

    Args:
        plugin_cls (type[Plugin]): 插件主类
        api_name (str | list[str], optional): 如果将插件作为 API 插件, 该参数为 API 名, 可以有多个
        api_version (VERSION, optional): 插件的 API 版本

    Raises:
        NotValidPluginError: 插件主类必须继承 Plugin 类

    Returns:
        Plugin: 插件主类
    """
    global __cached_frame
    try:
        if not Plugin.__subclasscheck__(plugin_cls):
            raise NotValidPluginError(f"插件主类必须继承 Plugin 类 而不是 {plugin_cls}")
    except TypeError as exc:
        raise NotValidPluginError(
            f"插件主类必须继承 Plugin 类 而不是 {plugin_cls.__class__}"
        ) from exc
    if __cached_frame is None:
        help(plugin_cls)
        exit()
    plugin_ins = plugin_cls(__cached_frame)
    if api_name:
        if isinstance(api_name, str):
            plugin_ins._api_names = [api_name]
        else:
            plugin_ins._api_names = api_name
        if api_version:
            plugin_ins._api_ver = api_version
        else:
            plugin_ins._api_ver = plugin_cls.version
    return plugin_ins


def help(plugin: type[Plugin]) -> None:
    """
    查看插件帮助.
    常用于查看 get_plugin_api() 方法获取到的插件实例的帮助.
    """
    plugin_docs = "<plugins.help>: " + plugin.name + "开放的 API 接口说明:\n"
    for attr_name, attr in plugin.__dict__.items():
        if not attr_name.startswith("__") and attr.__doc__ is not None:
            plugin_docs += (
                "\n §a" + attr_name + ":§f\n    " + attr.__doc__.replace("\n", "\n    ")
            )
    fmts.clean_print(plugin_docs)


def read_plugins(plugin_grp: "PluginGroup") -> None:
    """
    读取插件

    Args:
        plugin_grp (PluginGroup): 插件组
    """
    PLUGIN_PATH = os.path.join(TOOLDELTA_PLUGIN_DIR, TOOLDELTA_CLASSIC_PLUGIN)
    if PLUGIN_PATH not in sys.path:
        sys.path.append(PLUGIN_PATH)
    event_cbs.broadcast_listener.clear()
    event_cbs.dict_packet_funcs.clear()
    event_cbs.bytes_packet_funcs.clear()
    for plugin_dir in os.listdir(PLUGIN_PATH):
        if not plugin_is_enabled(plugin_dir):
            continue
        if os.path.isdir(os.path.join(PLUGIN_PATH, plugin_dir)):
            sys.path.append(os.path.join(PLUGIN_PATH, plugin_dir))
            load_plugin(plugin_grp, plugin_dir)
            if os.path.isfile(
                data_path := os.path.join(
                    "插件文件", TOOLDELTA_CLASSIC_PLUGIN, plugin_dir, "datas.json"
                )
            ):
                plugin_data = utils.safe_json.safe_json_load(data_path)
                plugin_grp.loaded_plugin_ids.append(plugin_data["plugin-id"])


def load_plugin(plugin_group: "PluginGroup", plugin_dirname: str) -> None | Plugin:
    """加载插件

    Args:
        plugin_group (PluginGroup): 插件组类
        plugin_dirname (str): 插件目录名
        hot_load (bool, optional): 是否热加载

    Raises:
        ValueError: 插件组未初始化读取
        ValueError: 插件组未绑定框架
        ValueError: 插件主类需要作者名
        ValueError: 插件主类需要作者名
        NotValidPluginError: 插件 不合法
        SystemExit: 插件名字不合法
        SystemExit: 插件配置文件报错
        SystemExit: 插件读取数据失败

    Returns:
        Union[None, Plugin]: 插件实例
    """
    global __cached_frame
    if isinstance(plugin_group, type(None)):
        raise ValueError("插件组未初始化读取")
    if isinstance(plugin_group.linked_frame, type(None)):
        raise ValueError("插件组未绑定框架")
    __cached_frame = plugin_group.linked_frame
    try:
        if os.path.isfile(
            os.path.join(
                "插件文件", TOOLDELTA_CLASSIC_PLUGIN, plugin_dirname, "__init__.py"
            )
        ):
            plugin_module = importlib.import_module(plugin_dirname)
            if plugin_module in loaded_plugin_modules:
                importlib.reload(plugin_module)
                mode_str = "重载"
            else:
                loaded_plugin_modules.append(plugin_module)
                mode_str = "载入"
        else:
            fmts.print_war(f"{plugin_dirname} 文件夹 未发现插件文件，跳过加载")
            return None
        plugin: Plugin | None = plugin_module.__dict__.get("entry")
        if not isinstance(plugin, Plugin):
            raise NotValidPluginError(
                "没有在最外层代码使用 entry = plugin_entry(YourPlugin) 语句注册插件"
            )
        if plugin.name is None or plugin.name == "":
            raise ValueError(f"插件主类 {plugin.__class__.__name__} 需要插件名")
        if len(plugin.version) != 3:
            raise NotValidPluginError(
                f"插件主类 {plugin.__class__.__name__} 的 version 属性需要是长度为 3 的元组, 如 (0, 0, 1)"
            )
        if plugin._api_names:
            # 此插件应该作为 API 插件
            for api_name in plugin._api_names:
                plugin_group.plugins_api[api_name] = plugin
        version_str = ".".join(map(str, plugin.version))
        plugin._plugin_group = plugin_group
        fmts.print_suc(
            f"已{mode_str}插件 §f{plugin.name}§b@{version_str} §a作者: §r{plugin.author}"
        )
        plugin_group.normal_plugin_loaded_num += 1
        return plugin
    except NotValidPluginError as err:
        fmts.print_err(f"插件 {plugin_dirname} 不合法：{err.args[0]}")
        raise SystemExit from err
    except cfg.ConfigError as err:
        fmts.print_err(f"插件 {plugin_dirname} 配置文件报错：{err}")
        fmts.print_err(
            "你也可以直接删除配置文件，重新启动 ToolDelta 以自动生成配置文件"
        )
        raise SystemExit from err
    except utils.safe_json.DataReadError as err:
        fmts.print_err(f"插件 {plugin_dirname} 读取数据失败：{err}")
    except SystemVersionException as err:
        fmts.print_err(f"插件 {plugin_dirname} 需要更高版本的 ToolDelta 加载：{err}")
        raise SystemExit
    except Exception as err:
        fmts.print_err(f"加载插件 {plugin_dirname} 出现问题，报错如下：")
        fmts.print_err("§c" + traceback.format_exc())
        raise SystemExit from err
    return None



LAUNCH_CFG: dict = {
    "启动器启动模式(请不要手动更改此项, 改为0可重置)": 0,
    "是否记录日志": True,
    "全局GitHub镜像": "",
    "插件市场源": "",
}
"默认登录配置"

LAUNCH_CFG_STD: dict = {
    "启动器启动模式(请不要手动更改此项, 改为0可重置)": int,
    "是否记录日志": bool,
    "全局GitHub镜像": str,
    "插件市场源": str,
}
"默认登录配置标准验证格式"

LAUNCHER_NEOMEGA_STD: dict = {
    "服务器号": int,
    "密码": str,
    "验证服务器地址(更换时记得更改fbtoken)": str,
}
LAUNCHER_NEOMEGA_DEFAULT: dict = {
    "服务器号": 0,
    "密码": "",
    "验证服务器地址(更换时记得更改fbtoken)": "",
}
LAUNCHER_NEOMEGARM_STD: dict = {
    "远程连接地址": str
}
LAUNCHER_NEOMEGARM_DEFAULT: dict = {
    "远程连接地址": "tcp://127.0.0.1:24020"
}
LAUNCHER_NEOMG2TD_DEFAULT: dict = {
    "服务器号": 0,
    "密码": "",
    "验证服务器地址(更换时记得更改fbtoken)": "",
}
LAUNCHER_NEOMG2TD_STD: dict = {
    "服务器号": int,
    "密码": str,
    "验证服务器地址(更换时记得更改fbtoken)": str,
}
LAUNCHER_NEOMG2TD_DEFAULT: dict = {
    "服务器号": 0,
    "密码": "",
    "验证服务器地址(更换时记得更改fbtoken)": "",
}
LAUNCHER_EULOGIST_DEFAULT: dict = {}
LAUNCHER_EULOGIST_STD: dict = {}
LAUNCHER_FATEARK_STD: dict = {
    "服务器号": int,
    "密码": str,
    "验证服务器地址(更换时记得更改fbtoken)": str,
}
LAUNCHER_FATEARK_DEFAULT: dict = {
    "服务器号": 0,
    "密码": "",
    "验证服务器地址(更换时记得更改fbtoken)": "",
}



import ctypes
import threading
import traceback
from typing import Any, TypeVar, ParamSpec, Generic
from collections.abc import Callable
from . import fmts


PT = ParamSpec("PT")


VT = TypeVar("VT")
RT = TypeVar("RT")
threads_list: list["ToolDeltaThread"] = []


class ThreadExit(SystemExit):
    """线程退出"""


class ToolDeltaThread(threading.Thread, Generic[PT, RT]):
    "简化 ToolDelta 子线程创建的 threading.Thread 的子类"

    SYSTEM = 0
    PLUGIN = 1
    PLUGIN_LOADER = 2

    def __init__(
        self,
        func: Callable[PT, RT],
        args: tuple = (),
        usage="",
        thread_level=PLUGIN,
        **kwargs,
    ):
        """新建一个 ToolDelta 子线程

        Args:
            func (Callable): 线程方法
            args (tuple, optional): 方法的参数项
            usage (str, optional): 线程的用途说明
            thread_level: 线程权限等级
            kwargs (dict, optional): 方法的关键词参数项
        """
        super().__init__(target=func)
        self.func = func
        self.daemon = True
        self.all_args = (args, kwargs)
        self.usage = usage or f"fn:{func.__name__}"
        self.stopping = False
        self._thread_level = thread_level
        self._ret: RT | None = None
        self._ret_exc = None
        self._stop_event = threading.Event()
        self.start()

    def run(self) -> None:
        """线程运行方法"""
        threads_list.append(self)
        try:
            args, kwargs = self.all_args
            self._ret = self.func(*args, **kwargs)
        except (SystemExit, ThreadExit) as e:
            self._ret_exc = e
            pass
        except ValueError as e:
            self._ret_exc = e
            if str(e) != "未连接到游戏":
                fmts.print_err(
                    f"线程 {self.usage or self.func.__name__} 出错:\n"
                    + traceback.format_exc()
                )
            else:
                fmts.print_war(f"线程 {self.usage} 因游戏断开连接被迫中断")
        except Exception as e:
            self._ret_exc = e
            fmts.print_err(
                f"线程 {self.usage or self.func.__name__} 出错:\n"
                + traceback.format_exc()
            )
        finally:
            threads_list.remove(self)
            self._stop_event.set()
            del self.all_args

    def stop(self) -> bool:
        """终止线程 注意: 不适合在有长时间sleep的线程内调用"""
        self.stopping = True
        thread_id = self.ident
        if thread_id is None:
            return True
        if not self.is_alive():
            return True
        # 也许不会出现问题了吧
        res = ctypes.pythonapi.PyThreadState_SetAsyncExc(
            ctypes.c_long(thread_id), ctypes.py_object(ThreadExit)
        )
        if res == 0:
            fmts.print_err(f"§c线程ID {thread_id} 不存在")
            return False
        elif res > 1:
            # 线程修改出问题了? 终止了奇怪的线程?
            # 回退修改
            ctypes.pythonapi.PyThreadState_SetAsyncExc(thread_id, None)
            fmts.print_err(f"§c终止线程 {self.name} 失败")
            return False
        self._stop_event.set()
        return True

    def block_get_result(self) -> RT:
        self._stop_event.wait()
        if self._ret_exc:
            raise self._ret_exc
        return self._ret  # type: ignore

    def block_get_result_with_timeout(self, timeout: float) -> RT | None:
        self._stop_event.wait(timeout)
        return self._ret


createThread = ToolDeltaThread


def get_threads_list() -> list["createThread"]:
    """返回使用 createThread 创建的全线程列表。"""
    return threads_list


def thread_func(usage: str, thread_level=ToolDeltaThread.PLUGIN):
    """
    在事件方法可能执行较久会造成堵塞时使用，方便快捷地创建一个新线程，例如:
    ```python
    @thread_func("一个会卡一分钟的线程")
    def on_inject(self):
        ...
    ```
    """

    def _recv_func(
        func: "Callable[PT, RT]",
    ) -> "Callable[PT, ToolDeltaThread[PT, RT]]":
        def thread_fun(*args: Any, **kwargs):
            return ToolDeltaThread(
                func,
                usage=usage,
                thread_level=thread_level,
                args=args,
                **kwargs,
            )

        thread_fun._orig = func
        thread_fun._usage = usage

        return thread_fun

    return _recv_func


def thread_gather(
    funcs_and_args: list[tuple[Callable[..., VT], tuple]],
) -> list[VT]:
    r"""
    使用回调线程作为并行器的伪异步执行器
    可以用于并发获取多个阻塞方法的返回
    ```
    >>> results = thread_gather([
        (requests.get, ("https://www.github.com",)),
        (time.sleep, (1,)),
        (sum, (1, 3, 5))
    ])
    >>> results
    [<Response 200>, None, 9]
    ```

    Args:
        funcs_and_args (list[tuple[Callable[..., VT], tuple]]): (方法, 参数) 的列表
    Returns

    Returns:
        list[VT]: 方法的返回 (按传入方法的顺序依次返回其结果)
    """
    res: list[Any] = [None] * len(funcs_and_args)
    finish_events = [threading.Event() for _ in range(len(funcs_and_args))]
    for i, (func, args) in enumerate(funcs_and_args):

        def _closet(_i: int, _func):
            def _cbfunc(*args):
                try:
                    assert isinstance(args, tuple), "传参必须是 tuple 类型"
                    res[_i] = _func(*args)
                finally:
                    finish_events[_i].set()

            return _cbfunc

        fun, usage = _closet(i, func), f"并行方法 {func.__name__}"
        ToolDeltaThread(fun, args, usage=usage)
    for evt in finish_events:
        evt.wait()
    return res


def force_stop_normal_threads():
    "强制终止非系统级线程, 仅系统内部调用"
    for i in threads_list:
        if i._thread_level != i.SYSTEM:
            fmts.print_suc(f"正在终止线程 {i.usage}  ", end="\r")
            res = i.stop()
            if res:
                fmts.print_suc(f"已终止线程 <{i.usage}>    ")
            else:
                fmts.print_suc(f"无法终止线程 <{i.usage}>  ")



import threading
from collections.abc import Callable

from ...constants import SysStatus, PacketIDS
from ...packets import Packet_CommandOutput
from ...mc_bytes_packet.base_bytes_packet import BaseBytesPacket
from ..packet_handler import PacketHandler
from ..types import UnreadyPlayer
from .neo_libs.blob_hash.blob_hash_holder import BlobHashHolder


class StandardFrame:
    """
    提供了标准的启动器框架，作为 ToolDelta 和游戏交互的接口
    新增的启动框架都应为此类的子类, 并实现和覆写所有 NotImplemented 的内容
    """

    # 启动器类型
    launch_type = "Original"

    def __init__(self) -> None:
        """实例化启动器框架"""
        self.dict_packet_handler = lambda pckType, pck: None
        self.bytes_packet_handler = lambda pckType, pck: None
        self.need_listen_packets: set[PacketIDS] = {
            PacketIDS.Text,
            PacketIDS.PlayerList,
            PacketIDS.UpdateAbilities,
            PacketIDS.CommandOutput,
        }
        self.exit_event = threading.Event()
        self.status: SysStatus = SysStatus.LOADING
        self._launcher_listeners: list[Callable[[], None]] = []

    def init(self):
        """初始化启动器框架"""

    def reload_listen_packets(self, listen_packets: set[PacketIDS]) -> None:
        """重载需要监听的数据包ID"""
        self.need_listen_packets = {
            PacketIDS.Text,
            PacketIDS.PlayerList,
            PacketIDS.UpdateAbilities,
            PacketIDS.CommandOutput,
        } | listen_packets

    def set_packet_listener(self, handler: PacketHandler):
        self.dict_packet_handler = handler.entrance_dict_packet
        self.bytes_packet_handler = handler.entrance_bytes_packet
        self.need_listen_packets |= handler.listen_packets

    def launch(self) -> None:
        """启动器启动

        Raises:
            SystemError: 无法启动此启动器
        """
        raise NotImplementedError

    def listen_launched(self, cbs: list[Callable[[], None]]) -> None:
        """设置监听启动器启动事件"""
        self._launcher_listeners.extend(cbs)

    def wait_crashed(self):
        self.exit_event.wait()

    def _exec_launched_listen_cbs(self):
        for cb in self._launcher_listeners:
            cb()

    def get_players_info(self) -> dict[str, UnreadyPlayer] | None:
        """
        获取启动器框架内存储的玩家信息
        如果框架无法存储, 则返回 None
        """
        raise NotImplementedError

    def get_bot_name(self) -> str:
        """获取机器人名字"""
        raise NotImplementedError

    def update_status(self, new_status: SysStatus) -> None:
        """更新启动器状态

        Args:
            new_status (int): 新的状态码
        """
        self.status = new_status
        if new_status in (SysStatus.NORMAL_EXIT, SysStatus.CRASHED_EXIT):
            self.exit_event.set()

    def sendcmd(
        self, cmd: str, waitForResp: bool = False, timeout: float = 30
    ) -> Packet_CommandOutput | None:
        """以玩家身份发送命令

        Args:
            cmd (str): 命令
            waitForResp (bool, optional): 是否等待结果
            timeout (int | float, optional): 超时时间

        Raises:
            NotImplementedError: 未实现此方法

        Returns:
            Optional[Packet_CommandOutput]: 返回命令结果
        """
        raise NotImplementedError

    def sendwscmd(
        self, cmd: str, waitForResp: bool = False, timeout: float = 30
    ) -> Packet_CommandOutput | None:
        """以 ws 身份发送命令

        Args:
            cmd (str): 命令
            waitForResp (bool, optional): 是否等待结果
            timeout (int | float, optional): 超时时间

        Raises:
            NotImplementedError: 未实现此方法

        Returns:
            Optional[Packet_CommandOutput]: 返回命令结果
        """
        raise NotImplementedError

    def sendwocmd(self, cmd: str) -> None:
        """以 wo 身份发送命令

        Args:
            cmd (str): 命令

        Raises:
            NotImplementedError: 未实现此方法
        """
        raise NotImplementedError

    def sendPacket(self, pckID: int, pck: dict | BaseBytesPacket) -> None:
        """发送数据包

        Args:
            pckID (int): 数据包 ID
            pck (dict | | BaseBytesPacket): 数据包内容

        Raises:
            NotImplementedError: 未实现此方法
        """
        raise NotImplementedError

    def blobHashHolder(self) -> BlobHashHolder:
        """blobHashHolder 返回 ToolDelta 的 Blob hash cache 缓存数据集的持有人

        Returns:
            BlobHashHolder: ToolDelta 的 Blob hash cache 缓存数据集的持有人

        Raises:
            NotImplementedError: 未实现此方法
        """
        raise NotImplementedError

    sendPacketJson = sendPacket



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: proto/listener.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'proto/listener.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import response_pb2 as proto_dot_response__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14proto/listener.proto\x12\x16\x66\x61teark.proto.listener\x1a\x14proto/response.proto\"\x16\n\x14ListenFateArkRequest\"\x16\n\x14ListenPacketsRequest\"-\n\x18ListenTypedPacketRequest\x12\x11\n\tpacket_id\x18\x01 \x01(\r\"\x1b\n\x19ListenPlayerChangeRequest\"\x13\n\x11ListenChatRequest\")\n\x19ListenCommandBlockRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"8\n\x06Output\x12\x10\n\x08msg_type\x18\x01 \x01(\t\x12\x0b\n\x03msg\x18\x02 \x01(\t\x12\x0f\n\x07\x65rr_msg\x18\x03 \x01(\t\"%\n\x06Packet\x12\n\n\x02id\x18\x01 \x01(\r\x12\x0f\n\x07payload\x18\x02 \x01(\t\"\x1e\n\x0cPlayerAction\x12\x0e\n\x06\x61\x63tion\x18\x01 \x01(\t\"\x17\n\x04\x43hat\x12\x0f\n\x07payload\x18\x01 \x01(\t2\xf6\x04\n\x0fListenerService\x12_\n\rListenFateArk\x12,.fateark.proto.listener.ListenFateArkRequest\x1a\x1e.fateark.proto.listener.Output0\x01\x12_\n\rListenPackets\x12,.fateark.proto.listener.ListenPacketsRequest\x1a\x1e.fateark.proto.listener.Packet0\x01\x12n\n\x11ListenTypedPacket\x12\x30.fateark.proto.listener.ListenTypedPacketRequest\x1a\'.fateark.proto.response.GeneralResponse\x12o\n\x12ListenPlayerChange\x12\x31.fateark.proto.listener.ListenPlayerChangeRequest\x1a$.fateark.proto.listener.PlayerAction0\x01\x12W\n\nListenChat\x12).fateark.proto.listener.ListenChatRequest\x1a\x1c.fateark.proto.listener.Chat0\x01\x12g\n\x12ListenCommandBlock\x12\x31.fateark.proto.listener.ListenCommandBlockRequest\x1a\x1c.fateark.proto.listener.Chat0\x01\x42!Z\x1f./proto/gen/listener;listenerpbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.listener_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\037./proto/gen/listener;listenerpb'
  _globals['_LISTENFATEARKREQUEST']._serialized_start=70
  _globals['_LISTENFATEARKREQUEST']._serialized_end=92
  _globals['_LISTENPACKETSREQUEST']._serialized_start=94
  _globals['_LISTENPACKETSREQUEST']._serialized_end=116
  _globals['_LISTENTYPEDPACKETREQUEST']._serialized_start=118
  _globals['_LISTENTYPEDPACKETREQUEST']._serialized_end=163
  _globals['_LISTENPLAYERCHANGEREQUEST']._serialized_start=165
  _globals['_LISTENPLAYERCHANGEREQUEST']._serialized_end=192
  _globals['_LISTENCHATREQUEST']._serialized_start=194
  _globals['_LISTENCHATREQUEST']._serialized_end=213
  _globals['_LISTENCOMMANDBLOCKREQUEST']._serialized_start=215
  _globals['_LISTENCOMMANDBLOCKREQUEST']._serialized_end=256
  _globals['_OUTPUT']._serialized_start=258
  _globals['_OUTPUT']._serialized_end=314
  _globals['_PACKET']._serialized_start=316
  _globals['_PACKET']._serialized_end=353
  _globals['_PLAYERACTION']._serialized_start=355
  _globals['_PLAYERACTION']._serialized_end=385
  _globals['_CHAT']._serialized_start=387
  _globals['_CHAT']._serialized_end=410
  _globals['_LISTENERSERVICE']._serialized_start=413
  _globals['_LISTENERSERVICE']._serialized_end=1043
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: proto/utils.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'proto/utils.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import response_pb2 as proto_dot_response__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11proto/utils.proto\x12\x13\x66\x61teark.proto.utils\x1a\x14proto/response.proto\"8\n\x11SendPacketRequest\x12\x11\n\tpacket_id\x18\x01 \x01(\x05\x12\x10\n\x08json_str\x18\x02 \x01(\t\"\x1f\n\x1dGetPacketNameIDMappingRequest\"(\n&GetClientMaintainedBotBasicInfoRequest\"&\n$GetClientMaintainedExtendInfoRequest2\xf4\x03\n\x0cUtilsService\x12]\n\nSendPacket\x12&.fateark.proto.utils.SendPacketRequest\x1a\'.fateark.proto.response.GeneralResponse\x12u\n\x16GetPacketNameIDMapping\x12\x32.fateark.proto.utils.GetPacketNameIDMappingRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x87\x01\n\x1fGetClientMaintainedBotBasicInfo\x12;.fateark.proto.utils.GetClientMaintainedBotBasicInfoRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x83\x01\n\x1dGetClientMaintainedExtendInfo\x12\x39.fateark.proto.utils.GetClientMaintainedExtendInfoRequest\x1a\'.fateark.proto.response.GeneralResponseB\x1bZ\x19./proto/gen/utils;utilspbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.utils_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\031./proto/gen/utils;utilspb'
  _globals['_SENDPACKETREQUEST']._serialized_start=64
  _globals['_SENDPACKETREQUEST']._serialized_end=120
  _globals['_GETPACKETNAMEIDMAPPINGREQUEST']._serialized_start=122
  _globals['_GETPACKETNAMEIDMAPPINGREQUEST']._serialized_end=153
  _globals['_GETCLIENTMAINTAINEDBOTBASICINFOREQUEST']._serialized_start=155
  _globals['_GETCLIENTMAINTAINEDBOTBASICINFOREQUEST']._serialized_end=195
  _globals['_GETCLIENTMAINTAINEDEXTENDINFOREQUEST']._serialized_start=197
  _globals['_GETCLIENTMAINTAINEDEXTENDINFOREQUEST']._serialized_end=235
  _globals['_UTILSSERVICE']._serialized_start=238
  _globals['_UTILSSERVICE']._serialized_end=738
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass
from tooldelta.internal.launch_cli.neo_libs.neo_conn import ThreadOmega


BLOCKING_DEADLINE_SECONDS = 30


@dataclass
class BaseBlobHashHolder:
    """BaseBlobHashHolder 是 Blob hash holder 的底层实现。

    Args:
        omega (ThreadOmega): omega ..
        is_disk_holder (bool): is_disk_holder 指示自身是否是镜像存档的持有人
        disk_holder_name (str): disk_holder_name 指示当前结点作为镜像存档的持有人的名字
    """

    omega: ThreadOmega
    is_disk_holder: bool
    disk_holder_name: str



from typing import TYPE_CHECKING
from dataclasses import dataclass
from tooldelta import game_utils
from .player_abilities import Abilities, upload_player_abilities

if TYPE_CHECKING:
    from ..maintainers.players import PlayerInfoMaintainer


@dataclass
class UnreadyPlayer:
    uuid: str
    unique_id: int
    name: str
    xuid: str
    platform_chat_id: str
    device_id: str | None
    build_platform: int
    online: bool = True
    abilities: Abilities | None = None

    def ready(self, _parent: "PlayerInfoMaintainer"):
        p = Player(
            _parent=_parent,
            uuid=self.uuid,
            unique_id=self.unique_id,
            name=self.name,
            xuid=self.xuid,
            device_id=None,
            runtime_id=None,
            platform_chat_id=self.platform_chat_id,
            build_platform=self.build_platform,
            online=self.online,
        )
        if self.abilities:
            p.setAbilities(self.abilities, upload=False)
        return p


@dataclass
class Player:
    _parent: "PlayerInfoMaintainer"
    uuid: str
    unique_id: int
    name: str
    xuid: str
    device_id: str | None
    runtime_id: int | None
    platform_chat_id: str
    build_platform: int
    online: bool = True

    def show(self, text: str):
        """
        对玩家显示聊天栏消息
        Args:
            text: 消息文本
        """
        self._parent.frame.get_game_control().say_to(self.name, text)

    def setTitle(self, title: str, sub_title: str = ""):
        """
        对玩家显示标题内容
        Args:
            title: 标题文本
            sub_title: 副标题文本, 默认为空
        """
        self._parent.frame.get_game_control().player_title(self.name, title)
        if sub_title.strip():
            self._parent.frame.get_game_control().player_subtitle(self.name, sub_title)

    def setActionbar(self, text: str):
        """
        对玩家显示行动栏文本
        Args:
            text: 动作条文本
        """
        self._parent.frame.get_game_control().player_actionbar(self.name, text)

    def getSelector(self):
        """
        获取包含玩家名的选择器

        等价于 `@a[name="{player.name}"]`
        Returns:
            str: 选择器
        """
        return f'@a[name="{self.name}"]'

    def getPos(self, timeout: float = 5) -> tuple[int, float, float, float]:
        """
        获取玩家坐标
        Args:
            timeout: 超时时间, 默认 5 秒
        Returns:
            tuple[int, float, float, float]: 维度ID, x, y, z
        """
        data = game_utils.getPos(self.name, timeout)
        dim = data["dimension"]
        pos = data["position"]
        x, y, z = pos["x"], pos["y"], pos["z"]
        return dim, x, y, z

    def getItemCount(self, item_id: str, item_data: int = -1, timeout: float = 5):
        """
        获取玩家背包中某一物品数量
        Args:
            item_id: 物品ID
            item_data: 物品数据值
            timeout: 超时时间, 默认 5 秒
        Returns:
            int: 物品数量
        """
        return game_utils.getItem(self.name, item_id, item_data)

    def getScore(self, scoreboard_name: str, timeout: float = 5):
        """
        获取玩家计分板分数
        Args:
            scoreboard_name: 计分板名称
            timeout: 超时时间, 默认 5 秒
        Returns:
            int: 计分板分数
        """
        return game_utils.getScore(scoreboard_name, self.getSelector(), timeout)

    def input(self, prompt: str = "", timeout: float = 30):
        """
        获取玩家聊天栏输入
        Args:
            prompt: 提示词
            timeout: 超时时间, 默认 30 秒
        Returns:
            str: 玩家输入
        """
        if prompt.strip():
            self.show(prompt)
        return game_utils.waitMsg(self.name, timeout)

    @property
    def abilities(self):
        """
        获取玩家能力
        Returns:
            Abilities: 玩家能力
        """
        return self._parent.get_player_ability(self)

    @abilities.setter
    def abilities(self, abilities: Abilities):
        self.setAbilities(abilities)

    def setAbilities(self, abilities: Abilities, upload=True):
        """
        设置玩家能力
        Args:
            abilities: 玩家能力
        """
        self._parent.player_abilities[self.unique_id] = abilities
        if upload:
            upload_player_abilities(
                self._parent.frame.get_game_control(), self.unique_id, abilities
            )

    def is_op(self):
        "玩家是否为管理员"
        return self.abilities.command_permissions >= 3

    @property
    def safe_name(self):
        "玩家可用于填充到指令内的名称"
        return '"' + self.name.replace("\\", "\\\\").replace('"', '\\"') + '"'

    def __hash__(self) -> int:
        return hash(self.unique_id)




from .player import Player, UnreadyPlayer
from .event_type import Chat, InternalBroadcast, FrameExit
from .player_abilities import Abilities

__all__ = ["Abilities", "Chat", "FrameExit", "InternalBroadcast", "Player", "UnreadyPlayer"]



"验证服务器的各种登录方法"

import getpass
import hashlib
import requests
import json

from .utils import fmts


def fblike_sign_login(url: str, api_urls_fmt: list[str]) -> str:
    """登录各大验证服务器获取 Token

    Raises:
        requests.exceptions.RequestException: 登录失败
    """
    login_api, new_api, main_api = (i % url for i in api_urls_fmt)
    while 1:
        hash_obj = hashlib.sha256()
        username = input(fmts.fmt_info("请输入账号: ", "§6 账号 "))
        hash_obj.update(
            getpass.getpass(fmts.fmt_info("请输入密码 (已隐藏):", "§6 密码 ")).encode()
        )
        password = hash_obj.hexdigest()
        auth_key = requests.get(url=new_api, timeout=5).text
        resp = requests.post(
            url=login_api,
            data=json.dumps(
                {
                    "client_public_key": "",
                    "server_code": "::DRY::",
                    "server_passcode": "::DRY::",
                    "username": username,
                    "password": password,
                },
                ensure_ascii=False,
            ),
            headers={
                "Content-Type": "application/json",
                "Authorization": f"Bearer {auth_key}",
            },
            timeout=5,
        )
        SUCCESS_CODE = 200
        if resp.status_code != SUCCESS_CODE:
            fmts.print_war(
                f"请求 Api 接口失败, 将自动使用 Token 登录 (状态码:{resp.status_code}, 返回值: {resp.text})"
            )
            raise requests.exceptions.RequestException("请求 Api 接口失败", resp)
        resp = resp.json()
        resp_msg = resp["message"]
        auth_succ = resp["success"]
        if not auth_succ:
            fmts.print_war(f"登录失败: {resp_msg}")
            continue
        break
    return resp["token"]



from typing import TYPE_CHECKING
from dataclasses import dataclass
from enum import IntEnum
from tooldelta.constants import PacketIDS

if TYPE_CHECKING:
    from tooldelta import GameCtrl
    from ..maintainers.players import PlayerInfoMaintainer
    from .player import Player


class Ability(IntEnum):
    AbilityBuild = 0
    AbilityMine = 1
    AbilityDoorsAndSwitches = 2
    AbilityOpenContainers = 3
    AbilityAttackPlayers = 4
    AbilityAttackMobs = 5
    AbilityOperatorCommands = 6
    AbilityTeleport = 7
    AbilityInvulnerable = 8
    AbilityFlying = 9
    AbilityMayFly = 10
    AbilityInstantBuild = 11
    AbilityLightning = 12
    AbilityFlySpeed = 13
    AbilityWalkSpeed = 14
    AbilityMuted = 15
    AbilityWorldBuilder = 16
    AbilityNoClip = 17
    AbilityPrivilegedBuilder = 18
    AbilityCount = 19


@dataclass
class Abilities:
    build: bool
    mine: bool
    doors_and_switches: bool
    open_containers: bool
    attack_players: bool
    attack_mobs: bool
    operator_commands: bool
    teleport: bool
    # invulnerable: bool
    # flying: bool
    # may_fly: bool
    # instant_build: bool
    # lightning: bool
    # fly_speed: float
    # walk_speed: float
    player_permissions: int
    command_permissions: int

    def auto_permission_level(self):
        if (
            self.build
            and self.mine
            and self.doors_and_switches
            and self.open_containers
            and self.attack_players
            and self.attack_mobs
            and self.operator_commands
            and self.teleport
        ):
            return 2 # 管理员权限
        elif (
            self.build
            and self.mine
            and self.doors_and_switches
            and self.open_containers
            and self.attack_players
            and self.attack_mobs
        ):
            return 1 # 普通玩家权限
        elif not (
            self.build
            or self.mine
            or self.doors_and_switches
            or self.open_containers
            or self.attack_mobs
            or self.attack_players
            or self.operator_commands
            or self.teleport
        ):
            return 0 # 访客权限
        else:
            return 3 # 自定义权限

    def marshal(self) -> int:
        return (
            (self.build << Ability.AbilityBuild)
            | (self.mine << Ability.AbilityMine)
            | (self.doors_and_switches << Ability.AbilityDoorsAndSwitches)
            | (self.open_containers << Ability.AbilityOpenContainers)
            | (self.attack_players << Ability.AbilityAttackPlayers)
            | (self.attack_mobs << Ability.AbilityAttackMobs)
            | (self.operator_commands << Ability.AbilityOperatorCommands)
            | (self.teleport << Ability.AbilityTeleport)
        )

    @classmethod
    def unmarshal(
        cls, abilities: int, player_permissions: int, command_permissions: int
    ):
        abilities = abilities & (
            (1 << Ability.AbilityBuild)
            | (1 << Ability.AbilityMine)
            | (1 << Ability.AbilityDoorsAndSwitches)
            | (1 << Ability.AbilityOpenContainers)
            | (1 << Ability.AbilityAttackPlayers)
            | (1 << Ability.AbilityAttackMobs)
            | (1 << Ability.AbilityOperatorCommands)
            | (1 << Ability.AbilityTeleport)
        )
        return cls(
            build=bool(abilities & (1 << Ability.AbilityBuild)),
            mine=bool(abilities & (1 << Ability.AbilityMine)),
            doors_and_switches=bool(abilities & (1 << Ability.AbilityDoorsAndSwitches)),
            open_containers=bool(abilities & (1 << Ability.AbilityOpenContainers)),
            attack_players=bool(abilities & (1 << Ability.AbilityAttackPlayers)),
            attack_mobs=bool(abilities & (1 << Ability.AbilityAttackMobs)),
            operator_commands=bool(abilities & (1 << Ability.AbilityOperatorCommands)),
            teleport=bool(abilities & (1 << Ability.AbilityTeleport)),
            player_permissions=player_permissions,
            command_permissions=command_permissions,
        )


def upload_player_abilities(
    pkt_sender: "GameCtrl", playerUniqueID: int, abilities: Abilities
):
    pkt_sender.sendPacket(
        PacketIDS.RequestPermissions,
        {
            "EntityUniqueID": playerUniqueID,
            "PermissionLevel": abilities.auto_permission_level(),
            "RequestedPermissions": abilities.marshal(),
        },
    )


def update_player_ability_from_ability_data(
    maintainer: "PlayerInfoMaintainer", player: "Player", ab_data: dict
):
    player_permissions = ab_data["PlayerPermissions"]
    command_permissions = ab_data["CommandPermissions"]
    for layer_data in ab_data["Layers"]:
        if layer_data["Type"] == 1:
            maintainer.player_abilities[player.unique_id] = Abilities.unmarshal(
                layer_data["Values"], player_permissions, command_permissions
            )



"""
客户端启动器框架
提供与游戏进行交互的标准接口
"""

import threading
from collections.abc import Callable

from ... import utils
from ...constants import SysStatus
from ...packets import Packet_CommandOutput
from ...mc_bytes_packet.base_bytes_packet import BaseBytesPacket
from ...utils import fmts
from ..types import UnreadyPlayer, Abilities
from .eulogist_libs import core_conn as eulogist_conn
from .standard_launcher import StandardFrame


class FrameEulogistLauncher(StandardFrame):
    # 启动器类型
    launch_type = "Eulogist"

    def __init__(self) -> None:
        """实例化启动器框架

        Args:
            serverNumber (int): 服务器号
            password (str): 服务器密码
            fbToken (str): 验证服务器 Token
            auth_server_url (str): 验证服务器地址
        """
        super().__init__()
        self.eulogist = eulogist_conn.Eulogist()
        self.need_listen_packets: set[int] = {9, 63, 79}
        self._launch_listeners: list[Callable[[], None]]
        self.exit_event = threading.Event()
        self.status: int = SysStatus.LOADING
        self.bot_name: str = ""

    def init(self):
        """初始化启动器框架"""

    def launch(self) -> SystemExit:
        """启动器启动

        Raises:
            SystemError: 无法启动此启动器
        """
        self.update_status(SysStatus.LAUNCHING)
        fmts.print_inf("正在从 10132 端口连接到赞颂者...")
        utils.createThread(
            self.eulogist.start, thread_level=utils.ToolDeltaThread.SYSTEM
        )
        self.eulogist.launch_event.wait()
        self.update_status(SysStatus.RUNNING)
        self.eulogist.packet_listener = self.packet_handler_parent
        self.eulogist.set_listen_server_packets(list(self.need_listen_packets))
        self._exec_launched_listen_cbs()
        self.eulogist.exit_event.wait()
        self.update_status(SysStatus.NORMAL_EXIT)
        return SystemExit("赞颂者和 ToolDelta 断开连接")

    def get_players_info(self) -> dict[str, UnreadyPlayer]:
        # TODO: Can't get PlatformChatID and BuildPlatformID
        return {
            k: UnreadyPlayer(
                uuid=v.uuid,
                unique_id=v.uniqueID,
                name=v.name,
                xuid=v.xuid,
                platform_chat_id="",
                device_id=None,
                build_platform=0,
                # NOTE: this is dangerous
                abilities=Abilities.unmarshal(
                    v.abilities["Layers"][0]["Abilities"],
                    v.abilities["PlayerPermissions"],
                    v.abilities["CommandPermissions"],
                ),
            )
            for k, v in self.eulogist.uqs.items()
        }

    def get_bot_name(self) -> str:
        """获取机器人名字"""
        if self.bot_name == "":
            self.bot_name = self.eulogist.bot_name
        return self.bot_name

    def update_status(self, new_status: int) -> None:
        """更新启动器状态

        Args:
            new_status (int): 新的状态码
        """
        self.status = new_status
        if new_status in (SysStatus.NORMAL_EXIT, SysStatus.CRASHED_EXIT):
            self.exit_event.set()

    def packet_handler_parent(self, pkt_type: int, pkt: dict) -> None:
        """数据包处理器

        Args:
            pkt_type (str): 数据包类型
            pkt (dict): 数据包内容

        Raises:
            ValueError: 还未连接到游戏
        """
        if not self.eulogist.connected:
            raise ValueError("还未连接到游戏")
        self.dict_packet_handler(pkt_type, pkt)

    def sendcmd(
        self, cmd: str, waitForResp: bool = False, timeout: float = 30
    ) -> Packet_CommandOutput | None:
        """以玩家身份发送命令

        Args:
            cmd (str): 命令
            waitForResp (bool, optional): 是否等待结果
            timeout (int | float, optional): 超时时间

        Raises:
            TimeoutError: 获取命令返回超时

        Returns:
            Packet_CommandOutput: 返回命令结果
        """
        if not waitForResp:
            self.eulogist.sendcmd(cmd)
        else:
            if res := self.eulogist.sendcmd_with_resp(cmd, timeout):
                return res
            else:
                raise TimeoutError("获取命令返回超时")

    def sendwscmd(
        self, cmd: str, waitForResp: bool = False, timeout: float = 30
    ) -> Packet_CommandOutput | None:
        """以 ws 身份发送命令

        Args:
            cmd (str): 命令
            waitForResp (bool, optional): 是否等待结果
            timeout (int | float, optional): 超时时间

        Raises:
            TimeoutError: 获取命令返回超时

        Returns:
            Packet_CommandOutput: 返回命令结果
        """
        if not waitForResp:
            self.eulogist.sendwscmd(cmd)
        else:
            if res := self.eulogist.sendwscmd_with_resp(cmd, timeout):
                return res
            else:
                raise TimeoutError("获取命令返回超时")

    def sendwocmd(self, cmd: str) -> None:
        """以 wo 身份发送命令

        Args:
            cmd (str): 命令

        """
        self.eulogist.sendwocmd(cmd)

    def sendPacket(self, pckID: int, pck: dict | BaseBytesPacket) -> None:
        """发送数据包

        Args:
            pckID (int): 数据包 ID
            pck (dict | BaseBytesPacket): 数据包内容

        """
        if type(pck) is not dict:
            raise Exception("sendPacket: Bytes packet is not supported")
        self.eulogist.sendPacket(pckID, pck)

    sendPacketJson = sendPacket

    def is_op(self, player: str) -> bool:
        """检查玩家是否为 OP

        Args:
            player (str): 玩家名

        Returns:
            bool: 是否为 OP
        """
        if player not in self.eulogist.uqs.keys():
            raise ValueError(f"玩家不存在: {player}")
        return self.eulogist.uqs[player].abilities["CommandPermissions"] >= 3



class BaseBytesPacket:
    def name(self) -> str:
        return ""

    def custom_packet_id(self) -> int:
        return -1

    def real_packet_id(self) -> int:
        return -1

    def encode(self) -> bytes:
        return b""

    def decode(self, bs: bytes):
        return



"""
客户端启动器框架
提供与游戏进行交互的标准接口
"""

from .standard_launcher import StandardFrame
from .neomega_access_point import FrameNeOmgAccessPoint
from .neomega_access_point_remote import FrameNeOmgAccessPointRemote
from .neomega_launcher import FrameNeOmegaLauncher
from .eulogist_client import FrameEulogistLauncher
from .fateark_access_point import FrameFateArk

FrameNeOmg = FrameNeOmgAccessPoint
FrameNeOmgRemote = FrameNeOmgAccessPointRemote
FrameEulogist = FrameEulogistLauncher

FB_LIKE_LAUNCHERS = (
    FrameNeOmegaLauncher
    | FrameNeOmgAccessPoint
    | FrameNeOmgAccessPointRemote
    | FrameEulogistLauncher
    | FrameFateArk
)
"类FastBuilder启动器框架"

ACCESS_POINT_LAUNCHERS = (
    FrameNeOmgAccessPoint,
    FrameNeOmegaLauncher,
    FrameFateArk
)
"接入点类型的启动框架"

LAUNCHERS = FB_LIKE_LAUNCHERS
"所有类型的启动框架"

__all__ = [
    "FB_LIKE_LAUNCHERS",
    "LAUNCHERS",
    "FrameEulogistLauncher",
    "FrameFateArk",
    "FrameNeOmegaLauncher",
    "FrameNeOmgAccessPoint",
    "FrameNeOmgAccessPointRemote",
    "StandardFrame"
]



"""提供了一些实用方法的类"""

import re
import threading
from typing import Any, TypeVar
from collections.abc import Callable

from .fmts import simple_fmt, print_war

__all__ = [
    "create_desperate_attr_class",
    "create_result_cb",
    "fill_list_index",
    "fuzzy_match",
    "remove_mc_color_code",
    "simple_assert",
    "simple_fmt",
    "split_list",
    "to_plain_name",
    "to_player_selector",
    "try_convert",
    "try_int",
]
MC_COLOR_CODE_REG = re.compile("§.")
FACTORY_TYPE = TypeVar("FACTORY_TYPE")
VT = TypeVar("VT")


def simple_assert(cond: Any, exc: Any) -> None:
    """相当于 assert cond, 但是可以自定义引发的异常的类型"""
    if not cond:
        raise exc


def try_int(arg: Any) -> int | None:
    """尝试将提供的参数化为 int 类型并返回，否则返回 None"""
    try:
        return int(arg)
    except Exception:
        return None


def try_convert(
    arg: Any, factory: Callable[[Any], FACTORY_TYPE]
) -> FACTORY_TYPE | None:
    """
    尝试将提供的参数交给传入的 factory 处理并返回

    Args:
        arg (Any): 参数
        factory (Callable[[Any], factory_type]): 处理器方法

    Returns:
        factory_type | None: 处理结果, 遇到 ValueError 则返回 None

    >>> try_convert("4.5", float)
    4.5
    >>> print(try_convert("3", bool))
    None
    """
    try:
        return factory(arg)
    except Exception:
        return None


def fuzzy_match(lst: list[str], sub: str, ignore_caps = True) -> list[str]:
    """
    模糊匹配列表内的字符串，可以用在诸如模糊匹配玩家名的用途

    参数:
        lst: list, 字符串列表
        sub: str, 需要匹配的字符串
        ignore_caps (bool, optional): 是否忽略大小写. Defaults to True.
    返回:
        list, 匹配结果
    """
    res = []
    if ignore_caps:
        for i in lst:
            if sub.lower() in i.lower():
                res.append(i)
    else:
        for i in lst:
            if sub in i:
                res.append(i)
    return res


def split_list(lst: list[VT], length: int) -> list[list[VT]]:
    """
    将列表进行块分割

    Args:
        lst (list[VT]): 传入列表
        length (int): 分割的单个列表的长度

    Returns:
        list[list[VT]]: 传出的被分割的列表
    """
    return [lst[i : i + length] for i in range(0, len(lst), length)]


def fill_list_index(lst: list[VT], default: list[VT]):
    """
    使用默认值填充列表。

    Args:
        lst (list[VT]): 待填充列表
        default (list[VT]): 默认的填充值 (补全待填充列表)
    """
    if len(lst) < len(default):
        lst.extend(default[len(lst) :])


def remove_mc_color_code(string: str):
    return MC_COLOR_CODE_REG.sub("", string)


def to_plain_name(name: str) -> str:
    """
    去除 网易版 Minecraft 的名字中的颜色代码
    可用于将 VIP 玩家名 转换为普通玩家名

    Args:
        name (str): 玩家名

    Returns:
        str: 去除颜色代码后的名字
    """
    if "§" in name:
        name = remove_mc_color_code(name)
    if name.count("<") > 1:
        # <<VIP名><玩家名>> -> 玩家名
        cached_str = ""
        words = []
        for char in name:
            if char == "<":
                cached_str = ""
            elif char == ">":
                words.append(cached_str)
            else:
                cached_str += char
        return words[-1]
    elif name.startswith("<") and name.endswith(">"):
        # <玩家名> -> 玩家名
        return name[1:-1]
    return name


def to_player_selector(playername: str) -> str:
    """
    将玩家名转换为目标选择器.
    >>> to_player_selector("123坐端正")
    '@a[name="123坐端正"]'
    >>> to_player_selector('@a[name="123坐端正"]') # 已有选择器不会再套选择器
    '@a[name="123坐端正"]'

    Args:
        playername (str): 玩家名

    Returns:
        str: 含玩家名的目标选择器
    """
    if not playername.startswith("@"):
        return f'@a[name="{playername}"]'
    else:
        # 很可能这就已经是目标选择器了
        return playername


def create_result_cb(typechecker: type[VT] = object):
    """
    获取一对回调锁
    Args:
        typechecker (type[VT], optional): 设置类型检测时回调函数返回值的类型, 无实际作用, Defaults to type[object].

    ```
    getter, setter = create_result_cb(str)

    cbs[special_id] = getter
    # 在这边等待回调...
    result = getter()

    # 与此同时, 在另一边...
    getting_data: str = ...
    if special_id in cbs.keys():
        cbs[special_id](getting_data)

    ```
    """
    ret: list[VT | None] = [None]
    lock = threading.Lock()
    lock.acquire()

    def getter(timeout=60.0) -> VT | None:
        lock.acquire(timeout=timeout)
        return ret[0]

    def setter(s: VT):
        ret[0] = s
        lock.release()

    return getter, setter


class DesperateFuncClass:
    def __init__(self):
        self._desperate_warn = False

    def __getattribute__(self, attr):
        if not object.__getattribute__(self, "_desperate_warn"):
            self._desperate_warn = True
            print_war(f"{type(self).__name__} 已被弃用。请查看文档以使用新方法。")
            #time.sleep(3)
        return object.__getattribute__(self, attr)


def create_desperate_attr_class(class_name, attrs: list[Callable | DesperateFuncClass]):
    new_class = type(
        class_name,
        (DesperateFuncClass,),
        {
            (f.__name__ if callable(f) else type(f).__name__): (
                staticmethod(f) if callable(f) else f
            )
            for f in attrs
        },
    )
    return new_class()



class PluginSkip(EOFError):
    """跳过插件加载"""


class NotValidPluginError(AssertionError):
    """仅加载插件时引发，不是合法插件的报错"""


class PluginAPINotFoundError(ModuleNotFoundError):
    """插件 API 未找到错误"""

    def __init__(self, name):
        self.name = name


class PluginAPIVersionError(ModuleNotFoundError):
    """插件 API 版本错误"""

    def __init__(self, name, m_ver, n_ver):
        self.name = name
        self.m_ver = m_ver
        self.n_ver = n_ver



from .packets import PacketIDS
from .minecraft import (
    TextType,
    ContainerType,
    PlayerActionType
)
from .tooldelta_cli import (
    TOOLDELTA_LOGO_mode,
    TOOLDELTA_LOGO,
    TOOLDELTA_PLUGIN_DIR,
    TOOLDELTA_CLASSIC_PLUGIN,
    TOOLDELTA_PLUGIN_CFG_DIR,
    PLUGIN_TYPE_MAPPING,
    TDSPECIFIC_MIRROR,
    TDSPECIFIC_GITHUB_DOWNLOAD_URL,
    TDREPO_URL,
    PLUGIN_MARKET_SOURCE_OFFICIAL,
    AUTH_SERVERS,
    FBLIKE_APIS,
    TOOLDELTA_PLUGIN_DATA_DIR,
    SysStatus,
)
from .tooldelta_cfg import (
    LAUNCH_CFG_STD,
    LAUNCH_CFG,
    LAUNCHER_NEOMG2TD_DEFAULT,
    LAUNCHER_NEOMEGA_DEFAULT,
    LAUNCHER_NEOMEGA_STD,
    LAUNCHER_NEOMG2TD_STD,
)

__all__ = [
    "AUTH_SERVERS",
    "FBLIKE_APIS",
    "LAUNCHER_NEOMEGA_DEFAULT",
    "LAUNCHER_NEOMEGA_STD",
    "LAUNCHER_NEOMG2TD_DEFAULT",
    "LAUNCHER_NEOMG2TD_STD",
    "LAUNCH_CFG",
    "LAUNCH_CFG_STD",
    "PLUGIN_MARKET_SOURCE_OFFICIAL",
    "PLUGIN_TYPE_MAPPING",
    "TDREPO_URL",
    "TDSPECIFIC_GITHUB_DOWNLOAD_URL",
    "TDSPECIFIC_MIRROR",
    "TOOLDELTA_CLASSIC_PLUGIN",
    "TOOLDELTA_LOGO",
    "TOOLDELTA_LOGO_mode",
    "TOOLDELTA_PLUGIN_CFG_DIR",
    "TOOLDELTA_PLUGIN_DATA_DIR",
    "TOOLDELTA_PLUGIN_DIR",
    "ContainerType",
    "PacketIDS",
    "PlayerActionType",
    "SysStatus",
    "TextType",
]



from tooldelta.internal.launch_cli.neo_libs.blob_hash.define import (
    BLOCKING_DEADLINE_SECONDS,
    BaseBlobHashHolder,
)
from tooldelta.internal.launch_cli.neo_libs.blob_hash.packet.define import (
    HashWithPosition,
    PayloadByHash,
)
from tooldelta.internal.launch_cli.neo_libs.blob_hash.packet.server_and_client import (
    ClientGetDiskHashPayload,
    ClientGetDiskHashPayloadResponse,
    ClientQueryDiskHashExist,
    ClientQueryDiskHashExistResponse,
    GetHashPayload,
    GetHashPayloadResponse,
    SetHolderRequest,
    SetHolderResponse,
)


class ClientFunction:
    """ClientFunction 基于 BaseBlobHashHolder 实现，为 Blob hash 客户端提供功能"""

    base_blob_hash_holder: BaseBlobHashHolder

    def __init__(self, base_blob_hash_holder: BaseBlobHashHolder):
        """
        Args:
            base_blob_hash_holder (BaseBlobHashHolder): Blob hash holder 的底层实现
        """
        self.base_blob_hash_holder = base_blob_hash_holder

    def set_holder_request(self) -> bool:
        """
        set_holder_request 请求服务者将客户端设置为镜像存档的持有人。
        一旦成功便不可撤销，持有人必须伴随服务者的存在而始终存在
        """
        request = SetHolderRequest()
        resp_bytes: bytes = self.base_blob_hash_holder.omega.soft_call_with_bytes(
            request.packet_name, request.encode(), timeout=BLOCKING_DEADLINE_SECONDS
        )  # type: ignore

        resp = SetHolderResponse()
        resp.decode(resp_bytes)

        if resp.success_states:
            self.base_blob_hash_holder.disk_holder_name = resp.holder_name
            self.base_blob_hash_holder.is_disk_holder = True
        return resp.success_states

    def get_hash_payload(
        self, hashes: list[HashWithPosition]
    ) -> dict[HashWithPosition, bytes]:
        """
        get_hash_payload 从服务者获取 hashes 对应的数据荷载。
        底层缓存集合会因此同时更新
        """
        mapping: dict[HashWithPosition, bytes] = {}

        reqeust = GetHashPayload(hashes)
        resp_bytes: bytes = self.base_blob_hash_holder.omega.soft_call_with_bytes(
            reqeust.packet_name, reqeust.encode(), timeout=BLOCKING_DEADLINE_SECONDS
        )  # type: ignore

        resp = GetHashPayloadResponse()
        resp.decode(resp_bytes)

        for i in resp.payload:
            payload = i.payload.tobytes()
            if self.base_blob_hash_holder.omega.update_blob_cache(i.hash.hash, payload):
                mapping[i.hash] = payload

        return mapping

    def query_disk_hash_exist(
        self, hashes: list[HashWithPosition]
    ) -> tuple[list[HashWithPosition], list[HashWithPosition]]:
        """
        query_disk_hash_exist 向镜像存档持有人发起检索请求，
        目的仅在于检索 hashes 是否命中镜像存档中的存储。

        返回的元组中的每个元素从左到右依次代表命中的缓存和
        未被命中的缓存
        """
        request = ClientQueryDiskHashExist(hashes)
        resp_bytes: bytes = self.base_blob_hash_holder.omega.soft_call_with_bytes(
            request.packet_name, request.encode(), timeout=BLOCKING_DEADLINE_SECONDS
        )  # type: ignore

        resp = ClientQueryDiskHashExistResponse()
        resp.decode(resp_bytes)

        hit: list[HashWithPosition] = []
        miss: list[HashWithPosition] = []

        for i in range(len(hashes)):
            if resp.states[i]:
                hit.append(hashes[i])
            else:
                miss.append(hashes[i])

        return hit, miss

    def get_disk_hash_payload(
        self, hashes: list[HashWithPosition]
    ) -> tuple[list[PayloadByHash], list[HashWithPosition]]:
        """
        get_disk_hash_payload 从镜像存档持有人请求 hashes 的二进制数据荷载。
        返回的元组中的每个元素从左到右依次代表命中的缓存和未被命中的缓存
        """
        request = ClientGetDiskHashPayload(hashes)
        resp_bytes: bytes = self.base_blob_hash_holder.omega.soft_call_with_bytes(
            request.packet_name, request.encode(), timeout=BLOCKING_DEADLINE_SECONDS
        )  # type: ignore

        resp = ClientGetDiskHashPayloadResponse()
        resp.decode(resp_bytes)

        mapping: dict[HashWithPosition, PayloadByHash] = {}
        for i in resp.payload:
            mapping[i.hash] = i

        hit: list[PayloadByHash] = []
        miss: list[HashWithPosition] = []

        for i in range(len(hashes)):
            if hashes[i] in mapping:
                hit.append(mapping[hashes[i]])
            else:
                miss.append(hashes[i])

        return hit, miss



from typing import TYPE_CHECKING, TypeVar
from collections.abc import Callable

from ...mc_bytes_packet.base_bytes_packet import BaseBytesPacket
from ...utils import fmts
from ...constants import PacketIDS
from ...internal.types import Player, Chat, InternalBroadcast, FrameExit
from ..basic import ON_ERROR_CB
from ..exceptions import (
    PluginAPINotFoundError,
    PluginAPIVersionError,
)

if TYPE_CHECKING:
    from .plugin_cls import Plugin
    from tooldelta.internal.packet_handler import (
        DictPacketListener,
        BytesPacketListener,
    )

T = TypeVar("T")
PluginEvent = tuple["Plugin", T]


on_preload_cbs: list[PluginEvent[Callable[[], None]]] = []
on_active_cbs: list[PluginEvent[Callable[[], None]]] = []
on_player_join_cbs: list[PluginEvent[Callable[[Player], None]]] = []
on_player_leave_cbs: list[PluginEvent[Callable[[Player], None]]] = []
on_chat_cbs: list[PluginEvent[Callable[[Chat], None]]] = []
on_frame_exit_cbs: list[PluginEvent[Callable[[FrameExit], None]]] = []
on_reloaded_cbs: list[PluginEvent[Callable[[], None]]] = []
dict_packet_funcs: dict[PacketIDS, list["DictPacketListener"]] = {}
bytes_packet_funcs: dict[PacketIDS, list["BytesPacketListener"]] = {}
broadcast_listener: dict[str, list[Callable[[InternalBroadcast], None]]] = {}


def reload():
    """系统调用, 重置所有处理函数"""
    on_preload_cbs.clear()
    on_active_cbs.clear()
    on_player_join_cbs.clear()
    on_player_leave_cbs.clear()
    on_chat_cbs.clear()
    on_frame_exit_cbs.clear()
    on_reloaded_cbs.clear()
    dict_packet_funcs.clear()
    bytes_packet_funcs.clear()
    broadcast_listener.clear()


def execute_preload(onerr: ON_ERROR_CB) -> None:
    """执行插件的二次初始化方法

    Args:
        onerr (Callable[[str, Exception, str], None], optional): 插件出错时的处理方法。Defaults to NON_FUNC.

    Raises:
        SystemExit: 缺少前置
        SystemExit: 前置版本过低
    """
    try:
        for p, func in on_preload_cbs:
            name = p.name
            func()
    except PluginAPINotFoundError as err:
        fmts.print_err(f"插件 {name} 需要包含该种接口的前置组件：{err.name}")
        raise SystemExit from err
    except PluginAPIVersionError as err:
        fmts.print_err(
            f"插件 {name} 需要该前置组件 {err.name} 版本：{err.m_ver}, 但是现有版本过低：{err.n_ver}"
        )
        raise SystemExit from err
    except Exception as err:
        onerr(name, err)
        raise SystemExit


def execute_active(onerr: ON_ERROR_CB) -> None:
    """执行插件的连接游戏后初始化方法

    Args:
        onerr (Callable[[str, Exception, str], None], optional): 插件出错时的处理方法
    """
    try:
        for p, func in on_active_cbs:
            name = p.name
            func()
    except Exception as err:
        onerr(name, err)


def execute_player_join(player: Player, onerr: ON_ERROR_CB) -> None:
    """执行玩家加入的方法

    Args:
        player (str): 玩家
        onerr (Callable[[str, Exception, str], None], optional): q 插件出错时的处理方法
    """
    try:
        for p, func in on_player_join_cbs:
            name = p.name
            func(player)
    except Exception as err:
        onerr(name, err)


def execute_chat(
    chat: Chat,
    onerr: ON_ERROR_CB,
) -> None:
    """执行玩家消息的方法

    Args:
        player (str): 玩家
        msg (str): 消息
        onerr (Callable[[str, Exception, str], None], optional): 插件出错时的处理方法
    """
    try:
        for p, func in on_chat_cbs:
            name = p.name
            func(chat)
    except Exception as err:
        onerr(name, err)


def execute_player_leave(player: Player, onerr: ON_ERROR_CB) -> None:
    """执行玩家离开的方法

    Args:
        player (str): 玩家
        onerr (Callable[[str, Exception, str], None], optional): 插件出错时的处理方法
    """
    try:
        for p, func in on_player_leave_cbs:
            name = p.name
            func(player)
    except Exception as err:
        onerr(name, err)


def execute_frame_exit(evt: FrameExit, onerr: ON_ERROR_CB):
    """执行框架退出的方法

    Args:
        onerr (Callable[[str, Exception, str], None], optional): 插件出错时的处理方法
    """
    try:
        for p, func in on_frame_exit_cbs:
            name = p.name
            func(evt)
    except Exception as err:
        onerr(name, err)


def execute_reloaded(onerr: ON_ERROR_CB):
    """执行插件重载的方法

    Args:
        onerr (Callable[[str, Exception, str], None], optional): 插件出错时的处理方法
    """
    try:
        for p, func in on_reloaded_cbs:
            name = p.name
            func()
    except Exception as err:
        onerr(name, err)


def execute_dict_packet_funcs(pktID: PacketIDS, pkt: dict, onerr: ON_ERROR_CB) -> bool:
    """处理字典类型的数据包监听器

    Args:
        pktID (int): 数据包 ID
        pkt (dict): 字典数据包

    Returns:
        bool: 是否处理成功
    """
    d = dict_packet_funcs.get(pktID)
    if d:
        try:
            for func in d:
                res = func(pkt)
                if res is True:
                    return True
        except Exception as err:
            onerr("插件方法:" + func.__name__, err)
    return False


def execute_bytes_packet_funcs(
    pktID: PacketIDS, pkt: BaseBytesPacket, onerr: ON_ERROR_CB
) -> bool:
    """处理二进制类型的数据包监听器

    Args:
        pktID (int): 数据包 ID
        pkt (BaseBytesPacket): 二进制数据包

    Returns:
        bool: 是否处理成功
    """
    d = bytes_packet_funcs.get(pktID)
    if d:
        try:
            for func in d:
                res = func(pkt)
                if res is True:
                    return True
        except Exception as err:
            onerr("插件方法:" + func.__name__, err)
    return False



import os
import sys
from pathlib import Path

PROTOC_PATH = Path(
    "C:/Users/25286/AppData/Local/Programs/Python/Python311/Lib/site-packages/grpc_tools/protoc.py"
)

this_dir = Path(os.path.dirname(__file__))

for file in os.listdir(this_dir / "proto"):
    if file.endswith(".proto"):
        os.system(
            f"{sys.executable} {PROTOC_PATH} --python_out={os.path.dirname(this_dir)} --grpc_python_out={os.path.dirname(this_dir)} -I {this_dir} {this_dir / 'proto' / file}"
        )

for file in os.listdir(this_dir):
    if file.endswith(".py") and file != "builder.py":
        with open(this_dir / file, encoding="utf-8") as f:
            content = f.read()
        content = content.replace("from proto import", "from . import")
        with open(this_dir / file, "w", encoding="utf-8") as f:
            f.write(content)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: proto/reversaler.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'proto/reversaler.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import response_pb2 as proto_dot_response__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16proto/reversaler.proto\x12\x18\x66\x61teark.proto.reversaler\x1a\x14proto/response.proto\"\x9b\x01\n\x18NewFateReversalerRequest\x12\x13\n\x0b\x61uth_server\x18\x01 \x01(\t\x12\x11\n\tuser_name\x18\x02 \x01(\t\x12\x15\n\ruser_password\x18\x03 \x01(\t\x12\x12\n\nuser_token\x18\x04 \x01(\t\x12\x13\n\x0bserver_code\x18\x05 \x01(\t\x12\x17\n\x0fserver_password\x18\x06 \x01(\t2\x89\x01\n\x15\x46\x61teReversalerService\x12p\n\x11NewFateReversaler\x12\x32.fateark.proto.reversaler.NewFateReversalerRequest\x1a\'.fateark.proto.response.GeneralResponseB%Z#./proto/gen/reversaler;reversalerpbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.reversaler_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z#./proto/gen/reversaler;reversalerpb'
  _globals['_NEWFATEREVERSALERREQUEST']._serialized_start=75
  _globals['_NEWFATEREVERSALERREQUEST']._serialized_end=230
  _globals['_FATEREVERSALERSERVICE']._serialized_start=233
  _globals['_FATEREVERSALERSERVICE']._serialized_end=370
# @@protoc_insertion_point(module_scope)



import os
import subprocess
from collections.abc import Callable
from grpc import RpcError

from ... import utils
from ...constants import SysStatus, PacketIDS
from ...packets import Packet_CommandOutput
from ...mc_bytes_packet.base_bytes_packet import BaseBytesPacket
from ...utils import fmts, urlmethod
from .standard_launcher import StandardFrame
from .fateark_libs import core_conn as fateark_core, utils as fateark_utils


class FrameFateArk(StandardFrame):
    launch_type = "FateArk"
    proc: subprocess.Popen

    def __init__(self) -> None:
        super().__init__()
        self.bot_name = ""
        self.command_output_cbs: dict[str, Callable] = {}

    def init(self) -> None:
        super().init()

    def set_launch_data(
        self, serverNumber: int, password: str, fbToken: str, auth_server_url: str
    ):
        self.serverNumber = serverNumber
        self.serverPassword = password
        self.fbToken = fbToken
        self.auth_server = auth_server_url

    def launch(self):
        self.update_status(SysStatus.LAUNCHING)
        free_port = urlmethod.get_free_port(19000)
        self.start_fateark_proc(free_port)
        self._proc_message_show_thread()
        fmts.print_suc(f"将在 {free_port} 端口启动 FateArk 接入点")
        fateark_core.connect(f"localhost:{free_port}")
        fmts.print_suc("FateArk 接入点进程已启动")
        self._message_show_thread()
        status, _, err_msg = fateark_core.login(
            self.auth_server, self.fbToken, str(self.serverNumber), self.serverPassword
        )
        if status != 0:
            self.update_status(SysStatus.CRASHED_EXIT)
            return SystemError(f"FateArk 登录失败: {err_msg}")
        self.update_status(SysStatus.RUNNING)
        fateark_core.set_listen_packets(set(self.need_listen_packets))
        self._packets_handler_thread()
        self._exec_launched_listen_cbs()
        self.wait_crashed()
        if self.status == SysStatus.NORMAL_EXIT:
            return SystemExit("正常退出")
        if self.status == SysStatus.CRASHED_EXIT:
            return Exception("接入点进程已崩溃")
        return SystemError("未知的退出状态")

    def start_fateark_proc(self, port: int):
        path = fateark_utils.get_bin_path()
        if not os.path.isfile(path):
            fmts.print_err(f"FateArk 接入点不存在: {path}")
            raise SystemExit
        self.proc = subprocess.Popen(
            [path, "-p", str(port)], stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )

    @utils.thread_func("FateArk 主输出线程", thread_level=utils.ToolDeltaThread.SYSTEM)
    def _message_show_thread(self):
        try:
            for msg_prefix, msg, err_msg in fateark_core.read_output():
                fmts.print_with_info(f"§b[{msg_prefix}]§r {msg}", "§b FARK ")
                if err_msg:
                    fmts.print_err("FateArk: " + err_msg)
                    self.update_status(SysStatus.CRASHED_EXIT)
                if msg_prefix == "Crash":
                    self.update_status(SysStatus.CRASHED_EXIT)
        except RpcError:
            fmts.print_inf("FateArk 输出通道已断开连接")

    @utils.thread_func(
        "FateArk 进程输出线程", thread_level=utils.ToolDeltaThread.SYSTEM
    )
    def _proc_message_show_thread(self):
        assert self.proc.stdout
        while 1:
            msg = self.proc.stdout.readline().decode("utf-8").strip()
            if msg == "":
                break
            fmts.print_with_info(msg, "§6 FARK ")
        fmts.print_inf("FateArk 进程已退出")

    @utils.thread_func(
        "FateArk 数据包处理线程", thread_level=utils.ToolDeltaThread.SYSTEM
    )
    def _packets_handler_thread(self):
        try:
            for id, packet in fateark_core.read_packet():
                self._packets_handler(id, packet)
        except RpcError as err:
            fmts.print_inf(f"FateArk 数据包处理通道已断开连接: {err}")
            self.update_status(SysStatus.CRASHED_EXIT)

    def _packets_handler(self, pkID: int, pk: dict):
        if pkID == PacketIDS.CommandOutput:
            self._command_output_handler(pk)
        self.dict_packet_handler(pkID, pk)

    def _command_output_handler(self, pk: dict):
        pkUUID = pk["CommandOrigin"]["UUID"]
        if pkUUID in self.command_output_cbs:
            self.command_output_cbs[pkUUID](pk)
        # else:
        #     fmts.print_war(f"命令没有对应回调: {pkUUID}")

    def check_avaliable(self):
        if self.status != SysStatus.RUNNING:
            raise ValueError("未连接到游戏")

    def sendcmd(
        self, cmd: str, waitForResp: bool = False, timeout: float = 30
    ) -> Packet_CommandOutput | None:
        """以玩家身份发送命令

        Args:
            cmd (str): 命令
            waitForResp (bool, optional): 是否等待结果
            timeout (int | float, optional): 超时时间

        Returns:
            Optional[Packet_CommandOutput]: 返回命令结果
        """
        self.check_avaliable()
        if waitForResp:
            ud = fateark_core.sendcmd_and_get_uuid(cmd)
            getter, setter = utils.create_result_cb(dict)
            self.command_output_cbs[ud] = setter
            res = getter(timeout)
            if res is None:
                raise TimeoutError("指令超时")
            return Packet_CommandOutput(res)
        return None

    def sendwscmd(
        self, cmd: str, waitForResp: bool = False, timeout: float = 30
    ) -> Packet_CommandOutput | None:
        self.check_avaliable()
        if waitForResp:
            ud = fateark_core.sendwscmd_and_get_uuid(cmd)
            getter, setter = utils.create_result_cb(dict)
            self.command_output_cbs[ud] = setter
            res = getter(timeout)
            if res is None:
                raise TimeoutError("指令超时")
            return Packet_CommandOutput(res)

    def sendwocmd(self, cmd: str):
        """以控制台身份发送命令

        Args:
            cmd (str): 命令

        """
        self.check_avaliable()
        fateark_core.sendwocmd(cmd)

    def sendPacket(self, pckID: int, pck: dict | BaseBytesPacket) -> None:
        """发送数据包

        Args:
            pckID (int): 数据包 ID
            pck (str | BaseBytesPacket): 数据包内容

        """
        if type(pck) is not dict:
            raise Exception("sendPacket: Bytes packet is not supported")
        fateark_core.sendPacket(pckID, pck)

    def get_players_info(self):
        uuids = fateark_core.get_online_player_uuids()
        return {
            fateark_core.get_unready_player(uuid).name: fateark_core.get_unready_player(
                uuid
            )
            for uuid in uuids
        }

    def get_bot_name(self) -> str:
        """获取机器人名字

        Returns:
            str: 机器人名字
        """
        self.check_avaliable()
        if not self.bot_name:
            self.bot_name = fateark_core.get_bot_name()
        return self.bot_name



from typing import Any
from dataclasses import dataclass
from .player import Player


class BasicEvent:
    @classmethod
    def load(cls, /): ...

@dataclass
class Chat(BasicEvent):
    player: Player
    msg: str

@dataclass
class InternalBroadcast(BasicEvent):
    evt_name: str
    data: Any

@dataclass
class FrameExit(BasicEvent):
    signal: int
    reason: str



"支持 mc 颜色代码的输出模块"

import datetime
import threading
import re
import colorama
from typing import Any

from .logger import publicLogger

colorama.init(autoreset=True)
MC_COLOR_CODE_REG = re.compile("§.")

INFO_NORMAL = "§f 信息 "
INFO_WARN = "§6 警告 "
INFO_ERROR = "§4 报错 "
INFO_FAIL = "§c 失败 "
INFO_SUCC = "§a 成功 "
INFO_LOAD = "§d 加载 "
print_lock = threading.RLock()

_original_print = print


def simple_fmt(kw: dict[str, Any], sub: str) -> str:
    """
    快速将字符串内按照给出的 dict 的键值对替换掉内容.

    参数:
        kw: Dict[str, Any], 键值对应替换的内容
        *args: str, 需要被替换的字符串

    示例:
        >>> my_color = "red"; my_item = "apple"
        >>> kw = {"[颜色]": my_color, "[物品]": my_item}
        >>> SimpleFmt(kw, "I like [颜色] [物品].")
        I like red apple.
    """
    for k, v in kw.items():
        if k in sub:
            sub = sub.replace(k, str(v))
    return sub


def colormode_replace(text: str, showmode=0) -> str:
    """颜色代码替换

    Args:
        text (str): 需要替换的字符串
        showmode (int, optional): 显示模式

    Returns:
        str: 替换后的字符串
    """
    # 1 = bg_color
    text = _strike(text)
    return (
        simple_fmt(
            {
                "§0": f"\033[{showmode};37;90m",
                "§1": f"\033[{showmode};37;34m",
                "§2": f"\033[{showmode};37;32m",
                "§3": f"\033[{showmode};37;36m",
                "§4": f"\033[{showmode};37;31m",
                "§5": f"\033[{showmode};37;35m",
                "§6": f"\033[{showmode};37;33m",
                "§7": f"\033[{showmode};37;90m",
                "§8": f"\033[{showmode};37;2m",
                "§9": f"\033[{showmode};37;94m",
                "§a": f"\033[{showmode};37;92m",
                "§b": f"\033[{showmode};37;96m",
                "§c": f"\033[{showmode};37;91m",
                "§d": f"\033[{showmode};37;95m",
                "§e": f"\033[{showmode};37;93m",
                "§f": f"\033[{showmode};37;1m",
                "§r": "\033[0m",
                "§u": "\033[4m",
                "§l": "\033[1m",
            },
            text,
        )
        + "\033[0m"
    )


def align(text: str, length: int = 15) -> str:
    """对齐字符串

    Args:
        text (str): 需要对齐的字符串
        length (int, optional): 对齐长度

    Returns:
        str: 对齐后的字符串
    """
    text_length = len(text)
    for char in text:
        if not char.isascii():
            text_length += 1
    return text + " " * (length - text_length)


def _strike(text: str) -> str:
    """删除线
    对于Unicode字符不适用

    Args:
        text (str): 需要删除线的字符串

    Returns:
        str: 删除线后的字符串
    """
    text_ok = ""
    strike_mode = False
    i = 0
    while i < len(text):
        char = text[i]
        try:
            if char == "§":
                if text[i + 1] == "S":
                    strike_mode = True
                    i += 2
                    continue
                if text[i + 1] == "r":
                    strike_mode = False
        except IndexError:
            pass
        if strike_mode:
            text_ok += "\u0336" + char
        else:
            text_ok += char
        i += 1
    return text_ok


def print_gradient(text, start_rgb, end_rgb):
    """使用ANSI转义码打印渐变文字"""
    result = []
    length = len(text)

    for i in range(length):
        ratio = i / (length - 1) if length > 1 else 0
        r = int(start_rgb[0] + (end_rgb[0] - start_rgb[0]) * ratio)
        g = int(start_rgb[1] + (end_rgb[1] - start_rgb[1]) * ratio)
        b = int(start_rgb[2] + (end_rgb[2] - start_rgb[2]) * ratio)
        result.append(f"\033[38;2;{r};{g};{b}m{text[i]}")

    return "".join(result) + "\033[0m"


def print_with_info(
    text: str, info: str = INFO_NORMAL, need_log: bool = True, **print_kwargs
):
    """输出带有信息的文本

    Args:
        text (str): 输出的文本
        info (str, optional): 输出的信息
        need_log (bool, optional): 是否需要记录日志
        **print_kwargs: 原 print 函数的参数

    Raises:
        AssertionError: 无法找到对应的颜色代码
    """
    with print_lock:
        if need_log:
            c_log(info, text)
        set_next_color = "§r"
        if "\n" in text:
            output_txts = []
            for text_line in str(text).split("\n"):
                if "§" in text_line:
                    try:
                        n = text_line.rfind("§")
                        _set_next_col_value = text_line[n : n + 2]
                        if set_next_color == -1:
                            raise AssertionError
                        set_next_color = _set_next_col_value
                    except Exception as exc:
                        string  = "输出信息时出现错误:\n"
                        string += "    %s\n"
                        string += "    原信息: %s"
                        _original_print(
                            string % (str(exc), text)
                        )
                output_txts.append(
                    datetime.datetime.now().strftime("[%H:%M] ")
                    + colormode_replace(info, 7)
                    + " "
                    + colormode_replace(set_next_color + text_line)
                )
            _original_print("\n".join(output_txts), **print_kwargs)
        else:
            _original_print(
                datetime.datetime.now().strftime("[%H:%M] ")
                + colormode_replace(info, 7)
                + " "
                + colormode_replace(text),
                **print_kwargs,
            )


def clean_print(text: str, **print_kwargs) -> None:
    """依照 mc 的颜色代码输出文本，可带有 print 函数的参数

    Args:
        text (str): 输出的文本
        **print_kwargs: 原 print 函数的参数
    """
    with print_lock:
        _original_print(colormode_replace(text), **print_kwargs)


def clean_fmt(text: str) -> str:
    """依照 mc 的颜色代码格式化文本

    Args:
        text (str): 需要格式化的文本

    Returns:
        str: 格式化后的文本
    """
    return colormode_replace(text)


def print(*args):
    print_inf(" ".join(str(i) for i in args))


def print_err(text: str, **print_kwargs) -> None:
    """输出错误信息

    Args:
        text (str): 输出的文本
    """
    print_with_info(f"§c{text}", INFO_ERROR, **print_kwargs)


def print_inf(text: str, **print_kwargs) -> None:
    """输出 INFO 信息

    Args:
        text (str): 输出的文本
    """
    print_with_info(f"{text}", INFO_NORMAL, **print_kwargs)


def print_suc(text: str, **print_kwargs) -> None:
    """输出成功信息

    Args:
        text (str): 输出的文本
    """
    print_with_info(f"§a{text}", INFO_SUCC, **print_kwargs)


def print_war(text: str, **print_kwargs) -> None:
    """输出警告信息

    Args:
        text (str): 输出的文本
    """
    print_with_info(f"§6{text}", INFO_WARN, **print_kwargs)


def print_load(text: str, **print_kwargs) -> None:
    """输出加载信息

    Args:
        text (str): 输出的文本
    """
    with print_lock:
        print_with_info(f"§d{text}", INFO_LOAD, **print_kwargs)


def fmt_info(text: str, info: str = "§f 信息 ") -> str:
    """格式化信息

    Args:
        text (str): 输出的文本
        info (str, optional): 输出的信息

    Raises:
        AssertionError: 无法找到对应的颜色代码

    Returns:
        str: 格式化后的信息
    """
    with print_lock:
        nextcolor = "§r"
        if "\n" in text:
            output_txts = []
            for text_line in str(text).split("\n"):
                if "§" in text_line:
                    try:
                        n = text_line.rfind("§")
                        _setnextcol = text_line[n : n + 2]
                        if nextcolor == -1:
                            raise AssertionError
                        nextcolor = _setnextcol
                    except Exception:
                        pass
                output_txts.append(
                    datetime.datetime.now().strftime("[%H:%M] ")
                    + colormode_replace(info, 7)
                    + " "
                    + colormode_replace(nextcolor + text_line)
                )
            return "\n".join(output_txts)
        return (
            datetime.datetime.now().strftime("[%H:%M] ")
            + colormode_replace(info, 7)
            + " "
            + colormode_replace(text)
        )


def c_log(inf: str, msg: str) -> None:
    """记录日志

    Args:
        inf (str): 信息
        msg (str): 记录的信息
    """
    with print_lock:
        for _g, _s in [
            ("§6 警告 ", "WARN"),
            ("§a 成功 ", "INFO"),
            ("§f 信息 ", "INFO"),
            ("§c 失败 ", "FAIL"),
            ("§4 报错 ", "ERROR"),
        ]:
            if inf == _g:
                inf = _s
                break
        msg = MC_COLOR_CODE_REG.sub("", msg)
        publicLogger.log_in(msg, inf.strip())


def get_ansi_rgb(r: int, g: int, b: int):
    return f"\033[38;2;{r};{g};{b}m"


def ansi_cls():
    _original_print("\033[2J")


def ansi_home():
    _original_print("\033[H")


def ansi_locate(x: int, y: int):
    _original_print(f"\033[{x};{y}H")


def ansi_save_screen():
    _original_print("\033[?47h")


def ansi_load_screen():
    _original_print("\033[?47l")



"""ToolDelta 启动器"""

import traceback

from .utils import fmts
from .frame import GameCtrl, ToolDelta

from .plugin_load.plugins import PluginGroup
from .internal.config_loader import ConfigLoader
from .internal.cmd_executor import ConsoleCmdManager
from .internal.maintainers.players import PlayerInfoMaintainer
from .internal.packet_handler import PacketHandler


tooldelta = ToolDelta()


def start_tool_delta() -> None:
    """启动 ToolDelta"""
    tooldelta.bootstrap()


def init_cfg_only() -> None:
    fmts.print_load("ToolDelta 正在以仅初始插件模式启动")
    try:
        tooldelta.cfg_loader = ConfigLoader(tooldelta)
        tooldelta.welcome()
        tooldelta.init_dirs()
        tooldelta.packet_handler = PacketHandler(tooldelta)
        tooldelta.cmd_manager = ConsoleCmdManager(tooldelta)
        tooldelta.players_maintainer = PlayerInfoMaintainer(tooldelta)
        tooldelta.plugin_group = PluginGroup(tooldelta)
        tooldelta.game_ctrl = GameCtrl(tooldelta)
        tooldelta.add_console_cmd_trigger = tooldelta.cmd_manager.add_console_cmd_trigger
        tooldelta.launcher = tooldelta.cfg_loader.load_tooldelta_cfg_and_get_launcher()
        tooldelta.launcher.set_packet_listener(tooldelta.packet_handler)
        tooldelta.game_ctrl.hook_launcher(tooldelta.launcher)
        tooldelta.game_ctrl.hook_packet_handler(tooldelta.packet_handler)
        tooldelta.plugin_group.hook_packet_handler(tooldelta.packet_handler)
        tooldelta.plugin_group.load_plugins()
        fmts.print_suc("ToolDelta 已初始化所有配置文件。")
    except (KeyboardInterrupt, SystemExit, EOFError) as err:
        if str(err):
            fmts.print_inf(f"ToolDelta 已关闭，退出原因：{err}")
        else:
            fmts.print_inf("ToolDelta 已关闭")
    except Exception:
        fmts.print_err(f"ToolDelta 运行过程中出现问题：{traceback.format_exc()}")


